@@ -16,16 +16,13 @@ import (
 	"net/http"
 	"net/url"
 	"os"
-	"os/signal"
 	"path"
 	"path/filepath"
 	"strconv"
 	"strings"
-	"syscall"
 
 	"github.com/chai2010/gettext-go/gettext"
 	"github.com/gorilla/websocket"
-	"golang.org/x/crypto/ssh/terminal"
 
 	"github.com/lxc/lxd/shared"
 )
@@ -186,16 +183,16 @@ func NewClient(config *Config, remote string) (*Client, error) {
 
 	c.name = remote
 
-	// TODO: Here, we don't support configurable local remotes, we only
-	// support the default local LXD at /var/lib/lxd/unix.socket.
 	if remote == "" {
-		c.BaseURL = "http://unix.socket"
-		c.BaseWSURL = "ws://unix.socket"
-		c.Transport = "unix"
-		c.http.Transport = &unixTransport
-		c.websocketDialer.NetDial = unixDial
-	} else if r, ok := config.Remotes[remote]; ok {
+		return nil, fmt.Errorf(gettext.Gettext("A remote name must be provided."))
+	}
+
+	if r, ok := config.Remotes[remote]; ok {
 		if r.Addr[0:5] == "unix:" {
+			if r.Addr == "unix://" {
+				r.Addr = fmt.Sprintf("unix:%s", shared.VarPath("unix.socket"))
+			}
+
 			c.BaseURL = "http://unix.socket"
 			c.BaseWSURL = "ws://unix.socket"
 			c.Transport = "unix"
@@ -215,7 +212,6 @@ func NewClient(config *Config, remote string) (*Client, error) {
 			c.http.Transport = &http.Transport{Dial: uDial}
 			c.websocketDialer.NetDial = uDial
 			c.Remote = &r
-			return &c, nil
 		} else {
 			certf, keyf, err := readMyCert()
 			if err != nil {
@@ -256,6 +252,7 @@ func NewClient(config *Config, remote string) (*Client, error) {
 	} else {
 		return nil, fmt.Errorf(gettext.Gettext("unknown remote name: %q"), remote)
 	}
+
 	if err := c.Finger(); err != nil {
 		return nil, err
 	}
@@ -506,6 +503,27 @@ func (c *Client) AmTrusted() bool {
 	return auth == "trusted"
 }
 
+func (c *Client) IsPublic() bool {
+	resp, err := c.GetServerConfig()
+	if err != nil {
+		return false
+	}
+
+	shared.Debugf("%s", resp)
+
+	jmap, err := resp.MetadataAsMap()
+	if err != nil {
+		return false
+	}
+
+	public, err := jmap.GetBool("public")
+	if err != nil {
+		return false
+	}
+
+	return public
+}
+
 func (c *Client) ListContainers() ([]shared.ContainerInfo, error) {
 	resp, err := c.get("containers?recursion=1")
 	if err != nil {
@@ -538,7 +556,7 @@ func (c *Client) CopyImage(image string, dest *Client, copy_aliases bool, aliase
 		"server":      c.BaseURL,
 		"fingerprint": fingerprint}
 
-	if info.Public == 0 {
+	if !shared.InterfaceToBool(info.Public) {
 		var operation string
 
 		resp, err := c.post("images/"+fingerprint+"/secret", nil, Async)
@@ -868,8 +886,11 @@ func (c *Client) GetImageInfo(image string) (*shared.ImageInfo, error) {
 	return &info, nil
 }
 
-func (c *Client) PutImageProperties(name string, p shared.ImageProperties) error {
-	body := shared.Jmap{"properties": p}
+func (c *Client) PutImageInfo(name string, p shared.BriefImageInfo) error {
+	body := shared.Jmap{}
+	body["public"] = p.Public
+	body["properties"] = p.Properties
+
 	_, err := c.put(fmt.Sprintf("images/%s", name), body, Sync)
 	return err
 }
@@ -1028,7 +1049,7 @@ func (c *Client) GetAlias(alias string) string {
 
 // Init creates a container from either a fingerprint or an alias; you must
 // provide at least one.
-func (c *Client) Init(name string, imgremote string, image string, profiles *[]string, ephem bool) (*Response, error) {
+func (c *Client) Init(name string, imgremote string, image string, profiles *[]string, config map[string]string, ephem bool) (*Response, error) {
 	var operation string
 	var tmpremote *Client
 	var err error
@@ -1067,7 +1088,7 @@ func (c *Client) Init(name string, imgremote string, image string, profiles *[]s
 			return nil, fmt.Errorf(gettext.Gettext("The image architecture is incompatible with the target server"))
 		}
 
-		if imageinfo.Public == 0 {
+		if !shared.InterfaceToBool(imageinfo.Public) {
 			resp, err := tmpremote.post("images/"+fingerprint+"/secret", nil, Async)
 			if err != nil {
 				return nil, err
@@ -1117,6 +1138,10 @@ func (c *Client) Init(name string, imgremote string, image string, profiles *[]s
 		body["profiles"] = *profiles
 	}
 
+	if config != nil {
+		body["config"] = config
+	}
+
 	if ephem {
 		body["ephemeral"] = ephem
 	}
@@ -1158,15 +1183,16 @@ func (c *Client) Init(name string, imgremote string, image string, profiles *[]s
 	return resp, nil
 }
 
-func (c *Client) LocalCopy(source string, name string, config map[string]string, profiles []string) (*Response, error) {
+func (c *Client) LocalCopy(source string, name string, config map[string]string, profiles []string, ephemeral bool) (*Response, error) {
 	body := shared.Jmap{
 		"source": shared.Jmap{
 			"type":   "copy",
 			"source": source,
 		},
-		"name":     name,
-		"config":   config,
-		"profiles": profiles,
+		"name":      name,
+		"config":    config,
+		"profiles":  profiles,
+		"ephemeral": ephemeral,
 	}
 
 	return c.post("containers", body, Async)
@@ -1180,10 +1206,21 @@ type secretMd struct {
 	Secret string `json:"secret"`
 }
 
-func (c *Client) Exec(name string, cmd []string, env map[string]string, stdin *os.File, stdout *os.File, stderr *os.File) (int, error) {
-	interactive := terminal.IsTerminal(int(stdin.Fd()))
+// Exec runs a command inside the LXD container. For "interactive" use such as
+// `lxc exec ...`, one should pass a controlHandler that talks over the control
+// socket and handles things like SIGWINCH. If running non-interactive, passing
+// a nil controlHandler will cause Exec to return when all of the command
+// output is sent to the output buffers.
+func (c *Client) Exec(name string, cmd []string, env map[string]string,
+	stdin io.ReadCloser, stdout io.WriteCloser,
+	stderr io.WriteCloser, controlHandler func(*Client, *websocket.Conn)) (int, error) {
 
-	body := shared.Jmap{"command": cmd, "wait-for-websocket": true, "interactive": interactive, "environment": env}
+	body := shared.Jmap{
+		"command":            cmd,
+		"wait-for-websocket": true,
+		"interactive":        controlHandler != nil,
+		"environment":        env,
+	}
 
 	resp, err := c.post(fmt.Sprintf("containers/%s/exec", name), body, Async)
 	if err != nil {
@@ -1195,57 +1232,13 @@ func (c *Client) Exec(name string, cmd []string, env map[string]string, stdin *o
 		return -1, err
 	}
 
-	if interactive {
+	if controlHandler != nil {
 		if wsControl, ok := md.FDs["control"]; ok {
-			go func() {
-				control, err := c.websocket(resp.Operation, wsControl)
-				if err != nil {
-					return
-				}
-
-				for {
-					width, height, err := terminal.GetSize(syscall.Stdout)
-					if err != nil {
-						continue
-					}
-
-					shared.Debugf("Window size is now: %dx%d", width, height)
-
-					w, err := control.NextWriter(websocket.TextMessage)
-					if err != nil {
-						shared.Debugf("Got error getting next writer %s", err)
-						break
-					}
-
-					msg := shared.ContainerExecControl{}
-					msg.Command = "window-resize"
-					msg.Args = make(map[string]string)
-					msg.Args["width"] = strconv.Itoa(width)
-					msg.Args["height"] = strconv.Itoa(height)
-
-					buf, err := json.Marshal(msg)
-					if err != nil {
-						shared.Debugf("Failed to convert to json %s", err)
-						break
-					}
-					_, err = w.Write(buf)
-
-					w.Close()
-					if err != nil {
-						shared.Debugf("Got err writing %s", err)
-						break
-					}
-
-					ch := make(chan os.Signal)
-					signal.Notify(ch, syscall.SIGWINCH)
-					sig := <-ch
-
-					shared.Debugf("Received '%s signal', updating window geometry.", sig)
-				}
-
-				closeMsg := websocket.FormatCloseMessage(websocket.CloseNormalClosure, "")
-				control.WriteMessage(websocket.CloseMessage, closeMsg)
-			}()
+			control, err := c.websocket(resp.Operation, wsControl)
+			if err != nil {
+				return -1, err
+			}
+			go controlHandler(c, control)
 		}
 
 		conn, err := c.websocket(resp.Operation, md.FDs["0"])
@@ -1255,20 +1248,22 @@ func (c *Client) Exec(name string, cmd []string, env map[string]string, stdin *o
 		shared.WebsocketSendStream(conn, stdin)
 		<-shared.WebsocketRecvStream(stdout, conn)
 	} else {
-		sources := []*os.File{stdin, stdout, stderr}
 		conns := make([]*websocket.Conn, 3)
 		dones := make([]chan bool, 3)
-		for i := 0; i < 3; i++ {
+
+		conns[0], err = c.websocket(resp.Operation, md.FDs[strconv.Itoa(0)])
+		if err != nil {
+			return -1, err
+		}
+		dones[0] = shared.WebsocketSendStream(conns[0], stdin)
+
+		outputs := []io.WriteCloser{stdout, stderr}
+		for i := 1; i < 3; i++ {
 			conns[i], err = c.websocket(resp.Operation, md.FDs[strconv.Itoa(i)])
 			if err != nil {
 				return -1, err
 			}
-
-			if i == 0 {
-				dones[i] = shared.WebsocketSendStream(conns[i], sources[i])
-			} else {
-				dones[i] = shared.WebsocketRecvStream(sources[i], conns[i])
-			}
+			dones[i] = shared.WebsocketRecvStream(outputs[i-1], conns[i])
 		}
 
 		/*
@@ -1288,7 +1283,7 @@ func (c *Client) Exec(name string, cmd []string, env map[string]string, stdin *o
 
 		// Once we're done, we explicitly close stdin, to signal the websockets
 		// we're done.
-		sources[0].Close()
+		stdin.Close()
 	}
 
 	// Now, get the operation's status too.
@@ -1314,6 +1309,13 @@ func (c *Client) Exec(name string, cmd []string, env map[string]string, stdin *o
 }
 
 func (c *Client) Action(name string, action shared.ContainerAction, timeout int, force bool) (*Response, error) {
+	if action == "start" {
+		current, err := c.ContainerStatus(name)
+		if err == nil && current.Status.StatusCode == shared.Frozen {
+			action = "unfreeze"
+		}
+	}
+
 	body := shared.Jmap{"action": action, "timeout": timeout, "force": force}
 	return c.put(fmt.Sprintf("containers/%s/state", name), body, Async)
 }
@@ -1427,7 +1429,7 @@ func (c *Client) GetMigrationSourceWS(container string) (*Response, error) {
 	return c.post(fmt.Sprintf("containers/%s", container), body, Async)
 }
 
-func (c *Client) MigrateFrom(name string, operation string, secrets map[string]string, config map[string]string, profiles []string, baseImage string) (*Response, error) {
+func (c *Client) MigrateFrom(name string, operation string, secrets map[string]string, config map[string]string, profiles []string, baseImage string, ephemeral bool) (*Response, error) {
 	source := shared.Jmap{
 		"type":       "migration",
 		"mode":       "pull",
@@ -1436,10 +1438,11 @@ func (c *Client) MigrateFrom(name string, operation string, secrets map[string]s
 		"base-image": baseImage,
 	}
 	body := shared.Jmap{
-		"source":   source,
-		"name":     name,
-		"config":   config,
-		"profiles": profiles,
+		"source":    source,
+		"name":      name,
+		"config":    config,
+		"profiles":  profiles,
+		"ephemeral": ephemeral,
 	}
 
 	return c.post("containers", body, Async)
@@ -1612,8 +1615,12 @@ func (c *Client) UpdateContainerConfig(container string, st shared.BriefContaine
 		"config":    st.Config,
 		"devices":   st.Devices,
 		"ephemeral": st.Ephemeral}
-	_, err := c.put(fmt.Sprintf("containers/%s", container), body, Async)
-	return err
+	resp, err := c.put(fmt.Sprintf("containers/%s", container), body, Async)
+	if err != nil {
+		return err
+	}
+
+	return c.WaitForSuccess(resp.Operation)
 }
 
 func (c *Client) ProfileCreate(p string) error {
