@@ -16,9 +16,9 @@ func (c *fingerCmd) showByDefault() bool {
 
 func (c *fingerCmd) usage() string {
 	return gettext.Gettext(
-		"Fingers the LXD instance to check if it is up and working.\n" +
-			"\n" +
-			"lxc finger <remote>\n")
+		`Fingers the LXD instance to check if it is up and working.
+
+lxc finger <remote>`)
 }
 
 func (c *fingerCmd) flags() {}
