@@ -36,11 +36,11 @@ var imageEditHelp string = gettext.Gettext(
 
 func (c *imageCmd) usage() string {
 	return gettext.Gettext(
-		`Manipulate container images
+		`Manipulate container images.
 
 lxc image import <tarball> [rootfs tarball] [target] [--public] [--created-at=ISO-8601] [--expires-at=ISO-8601] [--fingerprint=FINGERPRINT] [prop=value]
 
-lxc image copy [remote:]<image> <remote>: [--alias=ALIAS].. [--copy-alias] [--public]
+lxc image copy [remote:]<image> <remote>: [--alias=ALIAS].. [--copy-aliases] [--public]
 lxc image delete [remote:]<image>
 lxc image edit [remote:]<image>
 lxc image export [remote:]<image>
@@ -93,7 +93,7 @@ func doImageAlias(config *lxd.Config, args []string) error {
 		if len(args) > 2 {
 			remote, _ = config.ParseRemoteAndContainer(args[2])
 		} else {
-			remote = ""
+			remote, _ = config.ParseRemoteAndContainer("")
 		}
 		d, err := lxd.NewClient(config, remote)
 		if err != nil {
@@ -213,10 +213,10 @@ func (c *imageCmd) run(config *lxd.Config, args []string) error {
 		}
 		fmt.Printf(gettext.Gettext("Fingerprint: %s")+"\n", info.Fingerprint)
 		public := gettext.Gettext("no")
-		if info.Public == 1 {
+		if shared.InterfaceToBool(info) {
 			public = gettext.Gettext("yes")
 		}
-		fmt.Printf(gettext.Gettext("Size: %.2vMB")+"\n", float64(info.Size)/1024.0/1024.0)
+		fmt.Printf(gettext.Gettext("Size: %.2fMB")+"\n", float64(info.Size)/1024.0/1024.0)
 		arch, _ := shared.ArchitectureName(info.Architecture)
 		fmt.Printf(gettext.Gettext("Architecture: %s")+"\n", arch)
 		fmt.Printf(gettext.Gettext("Public: %s")+"\n", public)
@@ -268,6 +268,10 @@ func (c *imageCmd) run(config *lxd.Config, args []string) error {
 			}
 		}
 
+		if remote == "" {
+			remote = config.DefaultRemote
+		}
+
 		if imageFile == "" {
 			return errArgs
 		}
@@ -290,7 +294,7 @@ func (c *imageCmd) run(config *lxd.Config, args []string) error {
 		if len(args) > 1 {
 			remote, _ = config.ParseRemoteAndContainer(args[1])
 		} else {
-			remote = ""
+			remote, _ = config.ParseRemoteAndContainer("")
 		}
 
 		d, err := lxd.NewClient(config, remote)
@@ -329,12 +333,12 @@ func (c *imageCmd) run(config *lxd.Config, args []string) error {
 				return err
 			}
 
-			newdata := shared.ImageProperties{}
+			newdata := shared.BriefImageInfo{}
 			err = yaml.Unmarshal(contents, &newdata)
 			if err != nil {
 				return err
 			}
-			return d.PutImageProperties(image, newdata)
+			return d.PutImageInfo(image, newdata)
 		}
 
 		info, err := d.GetImageInfo(image)
@@ -342,7 +346,7 @@ func (c *imageCmd) run(config *lxd.Config, args []string) error {
 			return err
 		}
 
-		properties := info.Properties
+		properties := info.BriefInfo()
 		editor := os.Getenv("VISUAL")
 		if editor == "" {
 			editor = os.Getenv("EDITOR")
@@ -361,7 +365,7 @@ func (c *imageCmd) run(config *lxd.Config, args []string) error {
 			os.Remove(fname)
 			return err
 		}
-		f.Write([]byte(imageEditHelp))
+		f.Write([]byte(imageEditHelp + "\n"))
 		f.Write(data)
 		f.Close()
 		defer os.Remove(fname)
@@ -380,7 +384,7 @@ func (c *imageCmd) run(config *lxd.Config, args []string) error {
 			if err != nil {
 				return err
 			}
-			newdata := shared.ImageProperties{}
+			newdata := shared.BriefImageInfo{}
 			err = yaml.Unmarshal(contents, &newdata)
 			if err != nil {
 				fmt.Fprintf(os.Stderr, gettext.Gettext("YAML parse error %v")+"\n", err)
@@ -392,7 +396,7 @@ func (c *imageCmd) run(config *lxd.Config, args []string) error {
 
 				continue
 			}
-			err = d.PutImageProperties(image, newdata)
+			err = d.PutImageInfo(image, newdata)
 			break
 		}
 
@@ -407,6 +411,7 @@ func (c *imageCmd) run(config *lxd.Config, args []string) error {
 		if inName == "" {
 			return errArgs
 		}
+
 		d, err := lxd.NewClient(config, remote)
 		if err != nil {
 			return err
@@ -447,7 +452,7 @@ func (c *imageCmd) run(config *lxd.Config, args []string) error {
 			return err
 		}
 
-		properties := info.Properties
+		properties := info.BriefInfo()
 
 		data, err := yaml.Marshal(&properties)
 		fmt.Printf("%s", data)
@@ -500,13 +505,14 @@ func showImages(images []shared.ImageInfo) error {
 		fp := image.Fingerprint[0:12]
 		public := gettext.Gettext("no")
 		description := findDescription(image.Properties)
-		if image.Public == 1 {
+		if shared.InterfaceToBool(image.Public) {
 			public = gettext.Gettext("yes")
 		}
 		const layout = "Jan 2, 2006 at 3:04pm (MST)"
 		uploaded := time.Unix(image.UploadDate, 0).Format(layout)
 		arch, _ := shared.ArchitectureName(image.Architecture)
-		data = append(data, []string{shortest, fp, public, description, arch, uploaded})
+		size := fmt.Sprintf("%.2fMB", float64(image.Size)/1024.0/1024.0)
+		data = append(data, []string{shortest, fp, public, description, arch, size, uploaded})
 	}
 
 	table := tablewriter.NewWriter(os.Stdout)
@@ -517,6 +523,7 @@ func showImages(images []shared.ImageInfo) error {
 		gettext.Gettext("PUBLIC"),
 		gettext.Gettext("DESCRIPTION"),
 		gettext.Gettext("ARCH"),
+		gettext.Gettext("SIZE"),
 		gettext.Gettext("UPLOAD DATE")})
 	sort.Sort(ByName(data))
 	table.AppendBulk(data)
