@@ -24,6 +24,14 @@ func (a ByName) Swap(i, j int) {
 }
 
 func (a ByName) Less(i, j int) bool {
+	if a[i][0] == "" {
+		return false
+	}
+
+	if a[j][0] == "" {
+		return true
+	}
+
 	return a[i][0] < a[j][0]
 }
 
@@ -118,7 +126,7 @@ func listContainers(cinfos []shared.ContainerInfo, filters []string, listsnaps b
 			continue
 		}
 
-		if cstate.Status.StatusCode == shared.Running {
+		if cstate.Status.StatusCode == shared.Running || cstate.Status.StatusCode == shared.Frozen {
 			ipv4s := []string{}
 			ipv6s := []string{}
 			for _, ip := range cstate.Status.Ips {
@@ -184,9 +192,7 @@ func (c *listCmd) run(config *lxd.Config, args []string) error {
 
 	filters := []string{}
 
-	if len(args) == 0 {
-		remote = config.DefaultRemote
-	} else {
+	if len(args) != 0 {
 		filters = args
 		if strings.Contains(args[0], ":") {
 			remote, name = config.ParseRemoteAndContainer(args[0])
@@ -197,6 +203,10 @@ func (c *listCmd) run(config *lxd.Config, args []string) error {
 		}
 	}
 
+	if remote == "" {
+		remote = config.DefaultRemote
+	}
+
 	d, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
