@@ -174,12 +174,13 @@ func doProfileEdit(client *lxd.Client, p string) error {
 		os.Remove(fname)
 		return err
 	}
-	f.Write([]byte(profileEditHelp))
+	f.Write([]byte(profileEditHelp + "\n"))
 	f.Write(data)
 	f.Close()
 	defer os.Remove(fname)
 
 	for {
+		var err error
 		cmdParts := strings.Fields(editor)
 		cmd := exec.Command(cmdParts[0], append(cmdParts[1:], fname)...)
 		cmd.Stdin = os.Stdin
@@ -194,22 +195,21 @@ func doProfileEdit(client *lxd.Client, p string) error {
 			return err
 		}
 		newdata := shared.ProfileConfig{}
+
 		err = yaml.Unmarshal(contents, &newdata)
-		newdata.Name = p
+		if err == nil {
+			err = client.PutProfile(p, newdata)
+		}
+
 		if err != nil {
-			fmt.Fprintf(os.Stderr, gettext.Gettext("YAML parse error %v")+"\n", err)
+			fmt.Fprintf(os.Stderr, gettext.Gettext("Config parsing error: %s")+"\n", err)
 			fmt.Println(gettext.Gettext("Press enter to open the editor again"))
 			_, err := os.Stdin.Read(make([]byte, 1))
 			if err != nil {
 				return err
 			}
-
 			continue
 		}
-		err = client.PutProfile(p, newdata)
-		if err != nil {
-			return err
-		}
 		break
 	}
 	return nil
@@ -317,6 +317,15 @@ func doProfileSet(client *lxd.Client, p string, args []string) error {
 	} else {
 		value = args[1]
 	}
+
+	if !terminal.IsTerminal(syscall.Stdin) && value == "-" {
+		buf, err := ioutil.ReadAll(os.Stdin)
+		if err != nil {
+			return fmt.Errorf("Can't read from stdin: %s", err)
+		}
+		value = string(buf[:])
+	}
+
 	err := client.SetProfileConfigItem(p, key, value)
 	return err
 }
