@@ -91,7 +91,11 @@ func addServer(config *lxd.Config, server string, addr string, acceptCert bool,
 	if r_scheme == "unix" {
 		if addr[0:5] == "unix:" {
 			if addr[0:7] == "unix://" {
-				r_host = addr[8:]
+				if len(addr) > 8 {
+					r_host = addr[8:]
+				} else {
+					r_host = ""
+				}
 			} else {
 				r_host = addr[6:]
 			}
@@ -114,7 +118,7 @@ func addServer(config *lxd.Config, server string, addr string, acceptCert bool,
 	}
 
 	/* Actually add the remote */
-	config.Remotes[server] = lxd.RemoteConfig{Addr: addr, Public: public}
+	config.Remotes[server] = lxd.RemoteConfig{Addr: addr}
 
 	remote := config.ParseRemote(server)
 	c, err := lxd.NewClient(config, remote)
@@ -133,7 +137,9 @@ func addServer(config *lxd.Config, server string, addr string, acceptCert bool,
 		return err
 	}
 
-	if public {
+	if c.IsPublic() || public {
+		config.Remotes[server] = lxd.RemoteConfig{Addr: addr, Public: true}
+
 		if err := c.Finger(); err != nil {
 			return err
 		}
@@ -212,7 +218,7 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 		}
 
 		if config.DefaultRemote == args[1] {
-			config.DefaultRemote = ""
+			return fmt.Errorf(gettext.Gettext("can't remove the default remote"))
 		}
 
 		delete(config.Remotes, args[1])
@@ -276,11 +282,9 @@ func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 			return errArgs
 		}
 
-		if args[1] != "" {
-			_, ok := config.Remotes[args[1]]
-			if !ok {
-				return fmt.Errorf(gettext.Gettext("remote %s doesn't exist"), args[1])
-			}
+		_, ok := config.Remotes[args[1]]
+		if !ok {
+			return fmt.Errorf(gettext.Gettext("remote %s doesn't exist"), args[1])
 		}
 		config.DefaultRemote = args[1]
 
