@@ -18,6 +18,7 @@ import (
 	"reflect"
 	"regexp"
 	"sort"
+	"strconv"
 	"strings"
 	"syscall"
 	"time"
@@ -45,7 +46,23 @@ func extractInterfaceFromConfigName(k string) (string, error) {
 
 func validateRawLxc(rawLxc string) error {
 	for _, line := range strings.Split(rawLxc, "\n") {
+		// Ignore empty lines
+		if len(line) == 0 {
+			continue
+		}
+
+		// Ignore comments
+		if strings.HasPrefix(line, "#") {
+			continue
+		}
+
+		// Ensure the format is valid
 		membs := strings.SplitN(line, "=", 2)
+		if len(membs) != 2 {
+			return fmt.Errorf("invalid raw.lxc line: %s", line)
+		}
+
+		// Blacklist some keys
 		if strings.ToLower(strings.Trim(membs[0], " \t")) == "lxc.logfile" {
 			return fmt.Errorf("setting lxc.logfile is not allowed")
 		}
@@ -54,6 +71,15 @@ func validateRawLxc(rawLxc string) error {
 	return nil
 }
 
+func setConfigItem(c *containerLXD, key string, value string) error {
+	err := c.c.SetConfigItem(key, value)
+	if err != nil {
+		return fmt.Errorf("Failed to set LXC config: %s=%s", key, value)
+	}
+
+	return nil
+}
+
 // GenerateMacAddr generates a mac address from a string template:
 // e.g. "00:11:22:xx:xx:xx" -> "00:11:22:af:3e:51"
 func generateMacAddr(template string) (string, error) {
@@ -132,6 +158,7 @@ type container interface {
 
 	IsPrivileged() bool
 	IsRunning() bool
+	IsFrozen() bool
 	IsEphemeral() bool
 	IsSnapshot() bool
 
@@ -165,6 +192,110 @@ type container interface {
 
 	DetachMount(m shared.Device) error
 	AttachMount(m shared.Device) error
+	AttachUnixDev(dev shared.Device) error
+	DetachUnixDev(dev shared.Device) error
+}
+
+// unmount and unlink any directories called $path/blk.$(mktemp)
+func unmountTempBlocks(path string) {
+	dents, err := ioutil.ReadDir(path)
+	if err != nil {
+		return
+	}
+	for _, f := range dents {
+		bpath := f.Name()
+		dpath := filepath.Join(path, bpath)
+		if !strings.HasPrefix(bpath, "blk.") {
+			continue
+		}
+		if err = syscall.Unmount(dpath, syscall.MNT_DETACH); err != nil {
+			shared.Log.Warn("Failed to unmount block device", log.Ctx{"error": err, "path": dpath})
+			continue
+		}
+		if err = os.Remove(dpath); err != nil {
+			shared.Log.Warn("Failed to unlink block device mountpoint", log.Ctx{"error": err, "path": dpath})
+			continue
+		}
+	}
+}
+
+func getMountOptions(m shared.Device) ([]string, bool, bool) {
+	opts := []string{}
+	readonly := false
+	if m["readonly"] == "1" || m["readonly"] == "true" {
+		readonly = true
+		opts = append(opts, "ro")
+	}
+	optional := false
+	if m["optional"] == "1" || m["optional"] == "true" {
+		optional = true
+		opts = append(opts, "optional")
+	}
+
+	return opts, readonly, optional
+}
+
+/*
+ * This is called at container startup to mount any block
+ * devices (since a container with idmap cannot do so)
+ */
+func mountTmpBlockdev(cntPath string, d shared.Device) ([]string, error) {
+	source := d["source"]
+	fstype, err := shared.BlockFsDetect(source)
+	if err != nil {
+		return []string{}, fmt.Errorf("Error setting up %s: %s", source, err)
+	}
+	opts, readonly, optional := getMountOptions(d)
+
+	// Mount blockdev into $containerdir/blk.$(mktemp)
+	fnam := fmt.Sprintf("blk.%s", strings.Replace(source, "/", "-", -1))
+	blkmnt := filepath.Join(cntPath, fnam)
+	syscall.Unmount(blkmnt, syscall.MNT_DETACH)
+	os.Remove(blkmnt)
+	if err = os.Mkdir(blkmnt, 0660); err != nil {
+		if optional {
+			shared.Log.Warn("Failed to create block device mount directory",
+				log.Ctx{"error": err, "source": source})
+			return []string{}, nil
+		}
+		return []string{}, fmt.Errorf("Unable to create mountpoint for blockdev %s: %s", source, err)
+	}
+	flags := 0
+	if readonly {
+		flags |= syscall.MS_RDONLY
+	}
+	if err = syscall.Mount(source, blkmnt, fstype, uintptr(flags), ""); err != nil {
+		if optional {
+			shared.Log.Warn("Failed to mount block device", log.Ctx{"error": err, "source": source})
+			return []string{}, nil
+		}
+		return []string{}, fmt.Errorf("Unable to prepare blockdev mount for %s: %s", source, err)
+	}
+
+	opts = append(opts, "bind")
+	sb, err := os.Stat(source)
+	if err == nil {
+		if sb.IsDir() {
+			opts = append(opts, "create=dir")
+		} else {
+			opts = append(opts, "create=file")
+		}
+	}
+	tgtpath := d["path"]
+	for len(tgtpath) > 0 && filepath.IsAbs(tgtpath) {
+		tgtpath = tgtpath[1:]
+	}
+	if len(tgtpath) == 0 {
+		if optional {
+			shared.Log.Warn("Invalid mount target", log.Ctx{"target": d["path"]})
+			return []string{}, nil
+		}
+		return []string{}, fmt.Errorf("Invalid mount target %s", d["path"])
+	}
+	mtab := fmt.Sprintf("%s %s %s %s 0 0", blkmnt, tgtpath, "none", strings.Join(opts, ","))
+	shared.Debugf("adding mount entry for %s: .%s.\n", d["source"], mtab)
+
+	return []string{"lxc.mount.entry", mtab}, nil
 }
 
 func containerLXDCreateAsEmpty(d *Daemon, name string,
@@ -254,7 +385,7 @@ func containerLXDCreateAsSnapshot(d *Daemon, name string,
 		// TODO - shouldn't we freeze for the duration of rootfs snapshot below?
 		if !sourceContainer.IsRunning() {
 			c.Delete()
-			return nil, fmt.Errorf("Container not running\n")
+			return nil, fmt.Errorf("Container not running")
 		}
 		opts := lxc.CheckpointOptions{Directory: stateDir, Stop: true, Verbose: true}
 		err = sourceContainer.Checkpoint(opts)
@@ -299,12 +430,6 @@ func containerLXDCreateInternal(
 
 	path := containerPathGet(name, args.Ctype == cTypeSnapshot)
 	if shared.PathExists(path) {
-		shared.Log.Error(
-			"The container already exists on disk",
-			log.Ctx{
-				"container": name,
-				"path":      path})
-
 		return nil, fmt.Errorf(
 			"The container already exists on disk, container: '%s', path: '%s'",
 			name,
@@ -377,6 +502,26 @@ func containerLXDCreateInternal(
 		return nil, err
 	}
 
+	idmap := c.IdmapSetGet()
+
+	var jsonIdmap string
+	if idmap != nil {
+		idmapBytes, err := json.Marshal(idmap.Idmap)
+		if err != nil {
+			c.Delete()
+			return nil, err
+		}
+		jsonIdmap = string(idmapBytes)
+	} else {
+		jsonIdmap = "[]"
+	}
+
+	err = c.ConfigKeySet("volatile.last_state.idmap", jsonIdmap)
+	if err != nil {
+		c.Delete()
+		return nil, err
+	}
+
 	return c, nil
 }
 
@@ -410,7 +555,7 @@ func containerLXDLoad(d *Daemon, name string) (container, error) {
 		baseConfig:   baseConfig,
 		baseDevices:  baseDevices}
 
-	s, err := storageForFilename(d, c.PathGet(""))
+	s, err := storageForFilename(d, shared.VarPath("containers", strings.Split(name, "/")[0]))
 	if err != nil {
 		shared.Log.Warn("Couldn't detect storage.", log.Ctx{"container": c.NameGet()})
 		c.Storage = d.Storage
@@ -452,85 +597,29 @@ func (c *containerLXD) init() error {
 
 	personality, err := shared.ArchitecturePersonality(c.architecture)
 	if err == nil {
-		if err := c.c.SetConfigItem("lxc.arch", personality); err != nil {
+		if err := setConfigItem(c, "lxc.arch", personality); err != nil {
 			return err
 		}
 	}
 
-	err = c.c.SetConfigItem("lxc.include", fmt.Sprintf("%s/%s.common.conf", templateConfDir, templateConfBase))
+	err = setConfigItem(c, "lxc.include", fmt.Sprintf("%s/%s.common.conf", templateConfDir, templateConfBase))
 	if err != nil {
 		return err
 	}
 
-	if !c.IsPrivileged() {
-		err = c.c.SetConfigItem("lxc.include", fmt.Sprintf("%s/%s.userns.conf", templateConfDir, templateConfBase))
-		if err != nil {
-			return err
-		}
-	}
-
-	if c.IsNesting() {
-		shared.Debugf("Setting up %s for nesting", c.name)
-		orig := c.c.ConfigItem("lxc.mount.auto")
-		auto := ""
-		if len(orig) == 1 {
-			auto = orig[0]
-		}
-		if !strings.Contains(auto, "cgroup") {
-			auto = fmt.Sprintf("%s %s", auto, "cgroup:mixed")
-			err = c.c.SetConfigItem("lxc.mount.auto", auto)
-			if err != nil {
-				return err
-			}
-		}
-		/*
-		 * mount extra /proc and /sys to work around kernel
-		 * restrictions on remounting them when covered
-		 */
-		err = c.c.SetConfigItem("lxc.mount.entry", "proc dev/.lxc/proc proc create=dir,optional")
-		if err != nil {
-			return err
-		}
-		err = c.c.SetConfigItem("lxc.mount.entry", "sys dev/.lxc/sys sysfs create=dir,optional")
-		if err != nil {
-			return err
-		}
-	}
-
-	if err := c.c.SetConfigItem("lxc.rootfs", c.RootfsPathGet()); err != nil {
+	if err := setConfigItem(c, "lxc.rootfs", c.RootfsPathGet()); err != nil {
 		return err
 	}
-	if err := c.c.SetConfigItem("lxc.loglevel", "0"); err != nil {
+	if err := setConfigItem(c, "lxc.loglevel", "0"); err != nil {
 		return err
 	}
-	if err := c.c.SetConfigItem("lxc.utsname", c.NameGet()); err != nil {
+	if err := setConfigItem(c, "lxc.utsname", c.NameGet()); err != nil {
 		return err
 	}
-	if err := c.c.SetConfigItem("lxc.tty", "0"); err != nil {
+	if err := setConfigItem(c, "lxc.tty", "0"); err != nil {
 		return err
 	}
-	if err := setupDevLxdMount(c.c); err != nil {
-		return err
-	}
-
-	/*
-	 * Until stacked apparmor profiles are possible, we have to run nested
-	 * containers unconfined
-	 */
-	if aaEnabled {
-		if aaConfined() {
-			curProfile := aaProfile()
-			shared.Debugf("Running %s in current profile %s (nested container)", c.name, curProfile)
-			curProfile = strings.TrimSuffix(curProfile, " (enforce)")
-			if err := c.c.SetConfigItem("lxc.aa_profile", curProfile); err != nil {
-				return err
-			}
-		} else if err := c.c.SetConfigItem("lxc.aa_profile", AAProfileName(c)); err != nil {
-			return err
-		}
-	}
-
-	if err := c.c.SetConfigItem("lxc.seccomp", SeccompProfilePath(c)); err != nil {
+	if err := setupDevLxdMount(c); err != nil {
 		return err
 	}
 
@@ -545,6 +634,62 @@ func (c *containerLXD) init() error {
 		return err
 	}
 
+	if !c.IsPrivileged() || runningInUserns {
+		err = setConfigItem(c, "lxc.include", fmt.Sprintf("%s/%s.userns.conf", templateConfDir, templateConfBase))
+		if err != nil {
+			return err
+		}
+	}
+
+	if c.IsNesting() {
+		shared.Debugf("Setting up %s for nesting", c.name)
+		orig := c.c.ConfigItem("lxc.mount.auto")
+		auto := ""
+		if len(orig) == 1 {
+			auto = orig[0]
+		}
+		if !strings.Contains(auto, "cgroup") {
+			auto = fmt.Sprintf("%s %s", auto, "cgroup:mixed")
+			err = setConfigItem(c, "lxc.mount.auto", auto)
+			if err != nil {
+				return err
+			}
+		}
+		/*
+		 * mount extra /proc and /sys to work around kernel
+		 * restrictions on remounting them when covered
+		 */
+		err = setConfigItem(c, "lxc.mount.entry", "proc dev/.lxc/proc proc create=dir,optional")
+		if err != nil {
+			return err
+		}
+		err = setConfigItem(c, "lxc.mount.entry", "sys dev/.lxc/sys sysfs create=dir,optional")
+		if err != nil {
+			return err
+		}
+	}
+
+	/*
+	 * Until stacked apparmor profiles are possible, we have to run nested
+	 * containers unconfined
+	 */
+	if aaEnabled {
+		if aaConfined() {
+			curProfile := aaProfile()
+			shared.Debugf("Running %s in current profile %s (nested container)", c.name, curProfile)
+			curProfile = strings.TrimSuffix(curProfile, " (enforce)")
+			if err := setConfigItem(c, "lxc.aa_profile", curProfile); err != nil {
+				return err
+			}
+		} else if err := setConfigItem(c, "lxc.aa_profile", AAProfileFull(c)); err != nil {
+			return err
+		}
+	}
+
+	if err := setConfigItem(c, "lxc.seccomp", SeccompProfilePath(c)); err != nil {
+		return err
+	}
+
 	if err := c.setupMacAddresses(); err != nil {
 		return err
 	}
@@ -554,19 +699,13 @@ func (c *containerLXD) init() error {
 		c.devices[k] = v
 	}
 
-	/* now add the lxc.* entries for the configured devices */
-	if err := c.applyDevices(); err != nil {
-		return err
-	}
-
 	if !c.IsPrivileged() {
+		if c.daemon.IdmapSet == nil {
+			return fmt.Errorf("LXD doesn't have a uid/gid allocation. In this mode, only privileged containers are supported.")
+		}
 		c.idmapset = c.daemon.IdmapSet // TODO - per-tenant idmaps
 	}
 
-	if err := c.mountShared(); err != nil {
-		return err
-	}
-
 	if err := c.applyIdmapSet(); err != nil {
 		return err
 	}
@@ -596,7 +735,6 @@ func (c *containerLXD) RenderState() (*shared.ContainerState, error) {
 		Profiles:        c.profiles,
 		Config:          c.baseConfig,
 		ExpandedConfig:  c.config,
-		Userdata:        []byte{},
 		Status:          status,
 		Devices:         c.baseDevices,
 		ExpandedDevices: c.devices,
@@ -604,6 +742,144 @@ func (c *containerLXD) RenderState() (*shared.ContainerState, error) {
 	}, nil
 }
 
+func (c *containerLXD) insertMount(source, target, fstype string, flags int, options string) error {
+	pid := c.c.InitPid()
+	if pid == -1 { // container not running - we're done
+		return nil
+	}
+
+	// now live-mount
+	var tmpMount string
+	var err error
+	if shared.IsDir(source) {
+		tmpMount, err = ioutil.TempDir(shared.VarPath("shmounts", c.name), "lxdmount_")
+	} else {
+		f, err := ioutil.TempFile(shared.VarPath("shmounts", c.name), "lxdmount_")
+		if err == nil {
+			tmpMount = f.Name()
+			f.Close()
+		}
+	}
+	if err != nil {
+		return err
+	}
+
+	err = syscall.Mount(source, tmpMount, fstype, uintptr(flags), "")
+	if err != nil {
+		return err
+	}
+
+	mntsrc := filepath.Join("/dev/.lxd-mounts", filepath.Base(tmpMount))
+	// finally we need to move-mount this in the container
+	pidstr := fmt.Sprintf("%d", pid)
+	err = exec.Command(os.Args[0], "forkmount", pidstr, mntsrc, target).Run()
+	syscall.Unmount(tmpMount, syscall.MNT_DETACH) // in case forkmount failed
+	os.Remove(tmpMount)
+
+	return err
+}
+
+func (c *containerLXD) createUnixDevice(m shared.Device) (string, string, error) {
+	devname := m["path"]
+	if !filepath.IsAbs(devname) {
+		devname = filepath.Join("/", devname)
+	}
+
+	// target must be a relative path, so that lxc will DTRT
+	tgtname := m["path"]
+	for len(tgtname) > 0 && filepath.IsAbs(tgtname) {
+		tgtname = tgtname[1:]
+	}
+	if len(tgtname) == 0 {
+		return "", "", fmt.Errorf("Failed to interpret path: %s", devname)
+	}
+
+	var err error
+	var major, minor int
+	if m["major"] == "" && m["minor"] == "" {
+		major, minor, err = getDev(devname)
+		if err != nil {
+			return "", "", fmt.Errorf("Device does not exist: %s", devname)
+		}
+	} else if m["major"] == "" || m["minor"] == "" {
+		return "", "", fmt.Errorf("Both major and minor must be supplied for devices")
+	} else {
+		/* ok we have a major:minor and need to create it */
+		major, err = strconv.Atoi(m["major"])
+		if err != nil {
+			return "", "", fmt.Errorf("Bad major %s in device %s", m["major"], m["path"])
+		}
+		minor, err = strconv.Atoi(m["minor"])
+		if err != nil {
+			return "", "", fmt.Errorf("Bad minor %s in device %s", m["minor"], m["path"])
+		}
+	}
+
+	name := strings.Replace(m["path"], "/", "-", -1)
+	devpath := path.Join(c.PathGet(""), name)
+	mode := os.FileMode(0660)
+	if m["type"] == "unix-block" {
+		mode |= syscall.S_IFBLK
+	} else {
+		mode |= syscall.S_IFCHR
+	}
+
+	if m["mode"] != "" {
+		tmp, err := devModeOct(m["mode"])
+		if err != nil {
+			return "", "", fmt.Errorf("Bad mode %s in device %s", m["mode"], m["path"])
+		}
+		mode = os.FileMode(tmp)
+	}
+
+	os.Remove(devpath)
+	if err := syscall.Mknod(devpath, uint32(mode), minor|(major<<8)); err != nil {
+		if shared.PathExists(devname) {
+			return devname, tgtname, nil
+		}
+		return "", "", fmt.Errorf("Failed to create device %s for %s: %s", devpath, m["path"], err)
+	}
+
+	if err := c.idmapset.ShiftFile(devpath); err != nil {
+		// uidshift failing is weird, but not a big problem.  Log and proceed
+		shared.Debugf("Failed to uidshift device %s: %s\n", m["path"], err)
+	}
+
+	return devpath, tgtname, nil
+}
+
+func (c *containerLXD) setupUnixDev(m shared.Device) error {
+	source, target, err := c.createUnixDevice(m)
+	if err != nil {
+		return fmt.Errorf("Failed to setup device %s: %s", m["path"], err)
+	}
+
+	options, err := devGetOptions(m)
+	if err != nil {
+		return err
+	}
+
+	if c.c.Running() {
+		// insert mount from 'source' to 'target'
+		err := c.insertMount(source, target, "none", syscall.MS_BIND, options)
+		if err != nil {
+			return fmt.Errorf("Failed to add mount for device %s: %s", m["path"], err)
+		}
+
+		// add the new device cgroup rule
+		entry, err := deviceCgroupInfo(m)
+		if err != nil {
+			return fmt.Errorf("Failed to add cgroup rule for device %s: %s", m["path"], err)
+		}
+		if err := c.c.SetCgroupItem("devices.allow", entry); err != nil {
+			return fmt.Errorf("Failed to add cgroup rule %s for device %s: %s", entry, m["path"], err)
+		}
+	}
+
+	entry := fmt.Sprintf("%s %s none %s", source, target, options)
+	return c.c.SetConfigItem("lxc.mount.entry", entry)
+}
+
 func (c *containerLXD) Start() error {
 	if c.IsRunning() {
 		return fmt.Errorf("the container is already running")
@@ -627,8 +903,21 @@ func (c *containerLXD) Start() error {
 		return err
 	}
 
+	if err := c.mountShared(); err != nil {
+		return err
+	}
+
+	/*
+	 * add the lxc.* entries for the configured devices,
+	 * and create if necessary
+	 */
+	if err := c.applyDevices(); err != nil {
+		return err
+	}
+
 	f, err := ioutil.TempFile("", "lxd_lxc_startconfig_")
 	if err != nil {
+		unmountTempBlocks(c.PathGet(""))
 		c.StorageStop()
 		return err
 	}
@@ -636,6 +925,7 @@ func (c *containerLXD) Start() error {
 	if err = f.Chmod(0600); err != nil {
 		f.Close()
 		os.Remove(configPath)
+		unmountTempBlocks(c.PathGet(""))
 		c.StorageStop()
 		return err
 	}
@@ -643,12 +933,14 @@ func (c *containerLXD) Start() error {
 
 	err = c.c.SaveConfigFile(configPath)
 	if err != nil {
+		unmountTempBlocks(c.PathGet(""))
 		c.StorageStop()
 		return err
 	}
 
 	err = c.TemplateApply("start")
 	if err != nil {
+		unmountTempBlocks(c.PathGet(""))
 		c.StorageStop()
 		return err
 	}
@@ -658,6 +950,8 @@ func (c *containerLXD) Start() error {
 
 	lastIdmap, err := c.LastIdmapSetGet()
 	if err != nil {
+		unmountTempBlocks(c.PathGet(""))
+		c.StorageStop()
 		return err
 	}
 
@@ -665,6 +959,7 @@ func (c *containerLXD) Start() error {
 	if idmap != nil {
 		idmapBytes, err := json.Marshal(idmap.Idmap)
 		if err != nil {
+			unmountTempBlocks(c.PathGet(""))
 			c.StorageStop()
 			return err
 		}
@@ -678,6 +973,7 @@ func (c *containerLXD) Start() error {
 
 		if lastIdmap != nil {
 			if err := lastIdmap.UnshiftRootfs(c.RootfsPathGet()); err != nil {
+				unmountTempBlocks(c.PathGet(""))
 				c.StorageStop()
 				return err
 			}
@@ -685,6 +981,7 @@ func (c *containerLXD) Start() error {
 
 		if idmap != nil {
 			if err := idmap.ShiftRootfs(c.RootfsPathGet()); err != nil {
+				unmountTempBlocks(c.PathGet(""))
 				c.StorageStop()
 				return err
 			}
@@ -694,6 +991,7 @@ func (c *containerLXD) Start() error {
 	err = c.ConfigKeySet("volatile.last_state.idmap", jsonIdmap)
 
 	if err != nil {
+		unmountTempBlocks(c.PathGet(""))
 		c.StorageStop()
 		return err
 	}
@@ -707,6 +1005,7 @@ func (c *containerLXD) Start() error {
 		configPath).Run()
 
 	if err != nil {
+		unmountTempBlocks(c.PathGet(""))
 		c.StorageStop()
 		err = fmt.Errorf(
 			"Error calling 'lxd forkstart %s %s %s': err='%v'",
@@ -720,6 +1019,10 @@ func (c *containerLXD) Start() error {
 		containerWatchEphemeral(c.daemon, c)
 	}
 
+	if err != nil {
+		unmountTempBlocks(c.PathGet(""))
+		c.StorageStop()
+	}
 	return err
 }
 
@@ -755,6 +1058,10 @@ func (c *containerLXD) IsRunning() bool {
 	return c.c.Running()
 }
 
+func (c *containerLXD) IsFrozen() bool {
+	return c.StateGet() == "FROZEN"
+}
+
 func (c *containerLXD) Shutdown(timeout time.Duration) error {
 	if err := c.c.Shutdown(timeout); err != nil {
 		// Still try to unload the storage.
@@ -767,6 +1074,8 @@ func (c *containerLXD) Shutdown(timeout time.Duration) error {
 		return err
 	}
 
+	unmountTempBlocks(c.PathGet(""))
+
 	if err := AAUnloadProfile(c); err != nil {
 		return err
 	}
@@ -786,6 +1095,9 @@ func (c *containerLXD) Stop() error {
 		return err
 	}
 
+	// Clean up any mounts from previous runs
+	unmountTempBlocks(c.PathGet(""))
+
 	if err := AAUnloadProfile(c); err != nil {
 		return err
 	}
@@ -893,6 +1205,7 @@ func (c *containerLXD) Delete() error {
 		if err := c.Storage.ContainerDelete(c); err != nil {
 			return err
 		}
+		unmountTempBlocks(c.PathGet(""))
 	case cTypeSnapshot:
 		if err := c.Storage.ContainerSnapshotDelete(c); err != nil {
 			return err
@@ -912,6 +1225,12 @@ func (c *containerLXD) Delete() error {
 }
 
 func (c *containerLXD) Rename(newName string) error {
+	oldName := c.NameGet()
+
+	if !c.IsSnapshot() && !shared.ValidHostname(newName) {
+		return fmt.Errorf("Invalid container name")
+	}
+
 	if c.IsRunning() {
 		return fmt.Errorf("renaming of running container not allowed")
 	}
@@ -926,11 +1245,11 @@ func (c *containerLXD) Rename(newName string) error {
 		}
 	}
 
-	if err := dbContainerRename(c.daemon.db, c.NameGet(), newName); err != nil {
+	if err := dbContainerRename(c.daemon.db, oldName, newName); err != nil {
 		return err
 	}
 
-	results, err := dbContainerGetSnapshots(c.daemon.db, c.NameGet())
+	results, err := dbContainerGetSnapshots(c.daemon.db, oldName)
 	if err != nil {
 		return err
 	}
@@ -940,7 +1259,7 @@ func (c *containerLXD) Rename(newName string) error {
 		if err != nil {
 			shared.Log.Error(
 				"containerDeleteSnapshots: Failed to load the snapshotcontainer",
-				log.Ctx{"container": c.NameGet(), "snapshot": sname})
+				log.Ctx{"container": oldName, "snapshot": sname})
 
 			continue
 		}
@@ -949,7 +1268,7 @@ func (c *containerLXD) Rename(newName string) error {
 		if err := sc.Rename(newSnapshotName); err != nil {
 			shared.Log.Error(
 				"containerDeleteSnapshots: Failed to rename a snapshotcontainer",
-				log.Ctx{"container": c.NameGet(), "snapshot": sname, "err": err})
+				log.Ctx{"container": oldName, "snapshot": sname, "err": err})
 		}
 	}
 
@@ -1071,11 +1390,6 @@ func (c *containerLXD) ConfigReplace(newContainerArgs containerLXDArgs) error {
 	 */
 	preDevList := c.devices
 
-	/* Validate devices */
-	if err := validateConfig(c, newContainerArgs.Devices); err != nil {
-		return err
-	}
-
 	if err := c.applyConfig(newContainerArgs.Config); err != nil {
 		return err
 	}
@@ -1095,7 +1409,7 @@ func (c *containerLXD) ConfigReplace(newContainerArgs containerLXDArgs) error {
 	}
 
 	if err = dbContainerConfigInsert(tx, c.id, newContainerArgs.Config); err != nil {
-		shared.Debugf("Error inserting configuration for container %s", c.NameGet())
+		shared.Debugf("Error inserting configuration for container %s: %s", c.NameGet(), err)
 		tx.Rollback()
 		return err
 	}
@@ -1203,6 +1517,11 @@ func (c *containerLXD) ExportToTar(snap string, w io.Writer) error {
 		return fmt.Errorf("Cannot export a running container as image")
 	}
 
+	if err := c.StorageStart(); err != nil {
+		return err
+	}
+	defer c.StorageStop()
+
 	idmap, err := c.LastIdmapSetGet()
 	if err != nil {
 		return err
@@ -1379,79 +1698,79 @@ func (c *containerLXD) DetachMount(m shared.Device) error {
 	return exec.Command(os.Args[0], "forkumount", pidstr, m["path"]).Run()
 }
 
+/* This is called when adding a mount to a *running* container */
 func (c *containerLXD) AttachMount(m shared.Device) error {
 	dest := m["path"]
 	source := m["source"]
-
-	opts := ""
-	fstype := "none"
 	flags := 0
+
 	sb, err := os.Stat(source)
 	if err != nil {
 		return err
 	}
-	if sb.IsDir() {
-		flags |= syscall.MS_BIND
-		opts = "bind,create=dir"
-	} else {
-		if !shared.IsBlockdev(sb.Mode()) {
-			// Not sure if we want to try dealing with loopdevs, but
-			// since we'd need to deal with partitions i think not.
-			// We also might want to support file bind mounting, but
-			// this doesn't do that.
-			return fmt.Errorf("non-block device file not supported\n")
-		}
 
-		fstype, err = shared.BlockFsDetect(source)
+	opts, readonly, optional := getMountOptions(m)
+	if readonly {
+		flags |= syscall.MS_RDONLY
+	}
+
+	if shared.IsBlockdev(sb.Mode()) {
+		fstype, err := shared.BlockFsDetect(source)
 		if err != nil {
-			return fmt.Errorf("Unable to detect fstype for %s: %s\n", source, err)
+			if optional {
+				shared.Log.Warn("Failed to detect fstype for block device",
+					log.Ctx{"error": err, "source": source})
+				return nil
+			}
+			return fmt.Errorf("Unable to detect fstype for %s: %s", source, err)
 		}
-	}
 
-	// add a lxc.mount.entry = souce destination, in case of reboot
-	if m["readonly"] == "1" || m["readonly"] == "true" {
-		if opts == "" {
-			opts = "ro"
-		} else {
-			opts = opts + ",ro"
+		// Mount blockdev into $containerdir/blk.$(mktemp)
+		fnam := fmt.Sprintf("blk.%s", strings.Replace(source, "/", "-", -1))
+		blkmnt := filepath.Join(c.PathGet(""), fnam)
+		syscall.Unmount(blkmnt, syscall.MNT_DETACH)
+		os.Remove(blkmnt)
+		if err = os.Mkdir(blkmnt, 0660); err != nil {
+			if optional {
+				return nil
+			}
+			return fmt.Errorf("Unable to create mountpoint for blockdev %s: %s", source, err)
 		}
+		if err = syscall.Mount(source, blkmnt, fstype, uintptr(flags), ""); err != nil {
+			if optional {
+				return nil
+			}
+			return fmt.Errorf("Unable to prepare blockdev mount for %s: %s", source, err)
+		}
+
+		source = blkmnt
+		opts = append(opts, "create=dir")
+	} else if sb.IsDir() {
+		opts = append(opts, "create=dir")
+	} else {
+		opts = append(opts, "create=file")
 	}
-	optional := false
-	if m["optional"] == "1" || m["optional"] == "true" {
-		optional = true
-		opts = opts + ",optional"
+	opts = append(opts, "bind")
+	flags |= syscall.MS_BIND
+	optstr := strings.Join(opts, ",")
+
+	entry := fmt.Sprintf("%s %s %s %s 0 0", source, dest, "none", optstr)
+	if err := setConfigItem(c, "lxc.mount.entry", entry); err != nil {
+		if optional {
+			shared.Log.Warn("Failed to setup lxc mount for block device",
+				log.Ctx{"error": err, "source": source})
+		}
+		return fmt.Errorf("Failed to set up lxc mount entry for %s: %s", m["source"], err)
 	}
 
-	entry := fmt.Sprintf("%s %s %s %s 0 0", source, dest, fstype, opts)
-	if err := c.c.SetConfigItem("lxc.mount.entry", entry); err != nil {
-		return err
-	}
-
-	pid := c.c.InitPid()
-	if pid == -1 { // container not running - we're done
-		return nil
-	}
-
-	// now live-mount
-	tmpMount, err := ioutil.TempDir(shared.VarPath("shmounts", c.name), "lxdmount_")
+	err = c.insertMount(source, dest, "none", flags, optstr)
 	if err != nil {
-		return err
-	}
-
-	err = syscall.Mount(m["source"], tmpMount, fstype, uintptr(flags), "")
-	if err != nil {
-		return err
-	}
-
-	mntsrc := filepath.Join("/dev/.lxd-mounts", filepath.Base(tmpMount))
-	// finally we need to move-mount this in the container
-	pidstr := fmt.Sprintf("%d", pid)
-	err = exec.Command(os.Args[0], "forkmount", pidstr, mntsrc, m["path"]).Run()
-	syscall.Unmount(tmpMount, syscall.MNT_DETACH) // in case forkmount failed
-	os.Remove(tmpMount)
-
-	if err != nil && !optional {
-		return err
+		if optional {
+			shared.Log.Warn("Failed to insert mount for block device",
+				log.Ctx{"error": err, "source": m["source"]})
+			return nil
+		}
+		return fmt.Errorf("Failed to insert mount for %s: %s", m["source"], err)
 	}
 	return nil
 }
@@ -1469,16 +1788,16 @@ func (c *containerLXD) applyConfig(config map[string]string) error {
 				return err
 			}
 			if count != 1 || vint < 0 || vint > 65000 {
-				return fmt.Errorf("Bad cpu limit: %s\n", v)
+				return fmt.Errorf("Bad cpu limit: %s", v)
 			}
 			cpuset := fmt.Sprintf("0-%d", vint-1)
-			err = c.c.SetConfigItem("lxc.cgroup.cpuset.cpus", cpuset)
+			err = setConfigItem(c, "lxc.cgroup.cpuset.cpus", cpuset)
 		case "limits.memory":
-			err = c.c.SetConfigItem("lxc.cgroup.memory.limit_in_bytes", v)
+			err = setConfigItem(c, "lxc.cgroup.memory.limit_in_bytes", v)
 
 		default:
 			if strings.HasPrefix(k, "environment.") {
-				c.c.SetConfigItem("lxc.environment", fmt.Sprintf("%s=%s", strings.TrimPrefix(k, "environment."), v))
+				setConfigItem(c, "lxc.environment", fmt.Sprintf("%s=%s", strings.TrimPrefix(k, "environment."), v))
 			}
 
 			/* Things like security.privileged need to be propagated */
@@ -1700,7 +2019,7 @@ func (c *containerLXD) applyIdmapSet() error {
 	}
 	lines := c.idmapset.ToLxcString()
 	for _, line := range lines {
-		err := c.c.SetConfigItem("lxc.id_map", line+"\n")
+		err := setConfigItem(c, "lxc.id_map", line+"\n")
 		if err != nil {
 			return err
 		}
@@ -1721,14 +2040,19 @@ func (c *containerLXD) applyDevices() error {
 			continue
 		}
 
-		configs, err := deviceToLxc(d)
+		configs, err := deviceToLxc(c.PathGet(""), d)
 		if err != nil {
-			return fmt.Errorf("Failed configuring device %s: %s\n", name, err)
+			return fmt.Errorf("Failed configuring device %s: %s", name, err)
 		}
 		for _, line := range configs {
-			err := c.c.SetConfigItem(line[0], line[1])
+			err := setConfigItem(c, line[0], line[1])
 			if err != nil {
-				return fmt.Errorf("Failed configuring device %s: %s\n", name, err)
+				return fmt.Errorf("Failed configuring device %s: %s", name, err)
+			}
+		}
+		if d["type"] == "unix-block" || d["type"] == "unix-char" {
+			if err := c.setupUnixDev(d); err != nil {
+				return fmt.Errorf("Failed creating device %s: %s", d["name"], err)
 			}
 		}
 	}
@@ -1802,7 +2126,7 @@ func (c *containerLXD) tarStoreFile(linkmap map[uint64]string, offset int, tw *t
 
 	hdr.Uid, hdr.Gid, major, minor, ino, nlink, err = shared.GetFileStat(path)
 	if err != nil {
-		return fmt.Errorf("error getting file info: %s\n", err)
+		return fmt.Errorf("error getting file info: %s", err)
 	}
 
 	// unshift the id under /rootfs/ for unpriv containers
@@ -1831,17 +2155,17 @@ func (c *containerLXD) tarStoreFile(linkmap map[uint64]string, offset int, tw *t
 	// todo - handle xattrs
 
 	if err := tw.WriteHeader(hdr); err != nil {
-		return fmt.Errorf("error writing header: %s\n", err)
+		return fmt.Errorf("error writing header: %s", err)
 	}
 
 	if hdr.Typeflag == tar.TypeReg {
 		f, err := os.Open(path)
 		if err != nil {
-			return fmt.Errorf("tarStoreFile: error opening file: %s\n", err)
+			return fmt.Errorf("tarStoreFile: error opening file: %s", err)
 		}
 		defer f.Close()
 		if _, err := io.Copy(tw, f); err != nil {
-			return fmt.Errorf("error copying file %s\n", err)
+			return fmt.Errorf("error copying file %s", err)
 		}
 	}
 	return nil
@@ -1870,7 +2194,7 @@ func (c *containerLXD) mountShared() error {
 			return err
 		}
 	}
-	return c.c.SetConfigItem("lxc.mount.entry", entry)
+	return setConfigItem(c, "lxc.mount.entry", entry)
 }
 
 func (c *containerLXD) Checkpoint(opts lxc.CheckpointOptions) error {
