@@ -1,6 +1,7 @@
 package main
 
 import (
+	"fmt"
 	"net/http"
 
 	"github.com/gorilla/mux"
@@ -14,6 +15,10 @@ func containerDelete(d *Daemon, r *http.Request) Response {
 		return SmartError(err)
 	}
 
+	if c.IsRunning() {
+		return BadRequest(fmt.Errorf("container is running"))
+	}
+
 	rmct := func() error {
 		return c.Delete()
 	}
