@@ -6,7 +6,6 @@ import (
 	"net/http"
 
 	"github.com/gorilla/mux"
-	"github.com/lxc/lxd/lxd/migration"
 	"github.com/lxc/lxd/shared"
 )
 
@@ -28,17 +27,7 @@ func containerPost(d *Daemon, r *http.Request) Response {
 	}
 
 	if body.Migration {
-		lxc, err := c.LXContainerGet()
-		if err != nil {
-			return InternalError(err)
-		}
-
-		idmapset, err := c.IdmapSetGet()
-		if err != nil {
-			return InternalError(err)
-		}
-
-		ws, err := migration.NewMigrationSource(lxc, idmapset)
+		ws, err := NewMigrationSource(c)
 		if err != nil {
 			return InternalError(err)
 		}
