@@ -12,6 +12,7 @@ import (
 	"net"
 	"net/http"
 	"os"
+	"os/exec"
 	"runtime"
 	"strconv"
 	"strings"
@@ -24,6 +25,7 @@ import (
 	"github.com/gorilla/mux"
 	_ "github.com/mattn/go-sqlite3"
 	"github.com/stgraber/lxd-go-systemd/activation"
+	"github.com/syndtr/gocapability/capability"
 	"gopkg.in/tomb.v2"
 
 	"github.com/lxc/lxd"
@@ -32,6 +34,9 @@ import (
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
+var aaEnabled = true
+var runningInUserns = false
+
 const (
 	pwSaltBytes = 32
 	pwHashBytes = 64
@@ -300,17 +305,30 @@ func (d *Daemon) createCmd(version string, c Command) {
 }
 
 func (d *Daemon) SetupStorageDriver() error {
-	vgName, err := d.ConfigValueGet("storage.lvm_vg_name")
+	lvmVgName, err := d.ConfigValueGet("storage.lvm_vg_name")
 	if err != nil {
 		return fmt.Errorf("Couldn't read config: %s", err)
 	}
-	if vgName != "" {
+
+	zfsPoolName, err := d.ConfigValueGet("storage.zfs_pool_name")
+	if err != nil {
+		return fmt.Errorf("Couldn't read config: %s", err)
+	}
+
+	if lvmVgName != "" {
 		d.Storage, err = newStorage(d, storageTypeLvm)
 		if err != nil {
 			shared.Logf("Could not initialize storage type LVM: %s - falling back to dir", err)
 		} else {
 			return nil
 		}
+	} else if zfsPoolName != "" {
+		d.Storage, err = newStorage(d, storageTypeZfs)
+		if err != nil {
+			shared.Logf("Could not initialize storage type ZFS: %s - falling back to dir", err)
+		} else {
+			return nil
+		}
 	} else if d.BackingFs == "btrfs" {
 		d.Storage, err = newStorage(d, storageTypeBtrfs)
 		if err != nil {
@@ -361,7 +379,7 @@ func (d *Daemon) ListenAddresses() ([]string, error) {
 		localPort = shared.DefaultPort
 	}
 
-	if localHost == "0.0.0.0" || localHost == "::" {
+	if localHost == "0.0.0.0" || localHost == "::" || localHost == "[::]" {
 		ifaces, err := net.Interfaces()
 		if err != nil {
 			return addresses, err
@@ -397,26 +415,92 @@ func (d *Daemon) ListenAddresses() ([]string, error) {
 			}
 		}
 	} else {
-		ip := net.ParseIP(localHost)
-		if ip != nil && ip.IsGlobalUnicast() {
-			addresses = append(addresses, value)
-		}
+		addresses = append(addresses, value)
 	}
 
 	return addresses, nil
 }
 
+func bytesZero(x []byte) bool {
+	for _, b := range x {
+		if b != 0 {
+			return false
+		}
+	}
+	return true
+}
+
+func bytesEqual(x, y []byte) bool {
+	if len(x) != len(y) {
+		return false
+	}
+	for i, b := range x {
+		if y[i] != b {
+			return false
+		}
+	}
+	return true
+}
+
+func isZeroIP(x []byte) bool {
+	if x == nil {
+		return false
+	}
+
+	if bytesZero(x) {
+		return true
+	}
+
+	if len(x) != net.IPv6len {
+		return false
+	}
+
+	var v4InV6Prefix = []byte{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0xff, 0xff}
+	return bytesEqual(x[0:12], v4InV6Prefix) && bytesZero(x[12:])
+}
+
+func IpsEqual(ip1 net.IP, ip2 net.IP) bool {
+	if ip1.Equal(ip2) {
+		return true
+	}
+
+	/* the go std library Equal doesn't recognize [::] == 0.0.0.0, since it
+	 * tests for the ipv4 prefix, which isn't present in [::]. However,
+	 * they are in fact equal. Let's test for this case too.
+	 */
+	return isZeroIP(ip1) && isZeroIP(ip2)
+}
+
 func (d *Daemon) UpdateHTTPsPort(oldAddress string, newAddress string) error {
 	var sockets []Socket
 
+	if oldAddress == newAddress {
+		return nil
+	}
+
 	if oldAddress != "" {
-		_, _, err := net.SplitHostPort(oldAddress)
+		oldHost, oldPort, err := net.SplitHostPort(oldAddress)
 		if err != nil {
-			oldAddress = fmt.Sprintf("%s:%s", oldAddress, shared.DefaultPort)
+			oldHost = oldAddress
+			oldPort = shared.DefaultPort
 		}
 
+		// Strip brackets around IPv6 once we've gotten rid of the port
+		oldHost = strings.TrimLeft(oldHost, "[")
+		oldHost = strings.TrimRight(oldHost, "]")
+
 		for _, socket := range d.Sockets {
-			if socket.Socket.Addr().String() == oldAddress {
+			host, port, err := net.SplitHostPort(socket.Socket.Addr().String())
+			if err != nil {
+				host = socket.Socket.Addr().String()
+				port = shared.DefaultPort
+			}
+
+			// Strip brackets around IPv6 once we've gotten rid of the port
+			host = strings.TrimLeft(host, "[")
+			host = strings.TrimRight(host, "]")
+
+			if !shared.PathExists(host) && IpsEqual(net.ParseIP(host), net.ParseIP(oldHost)) && port == oldPort {
 				socket.Socket.Close()
 			} else {
 				sockets = append(sockets, socket)
@@ -429,7 +513,12 @@ func (d *Daemon) UpdateHTTPsPort(oldAddress string, newAddress string) error {
 	if newAddress != "" {
 		_, _, err := net.SplitHostPort(newAddress)
 		if err != nil {
-			newAddress = fmt.Sprintf("%s:%s", newAddress, shared.DefaultPort)
+			ip := net.ParseIP(newAddress)
+			if ip != nil && ip.To4() == nil {
+				newAddress = fmt.Sprintf("[%s]:%s", newAddress, shared.DefaultPort)
+			} else {
+				newAddress = fmt.Sprintf("%s:%s", newAddress, shared.DefaultPort)
+			}
 		}
 
 		tlsConfig, err := shared.GetTLSConfig(d.certf, d.keyf)
@@ -494,6 +583,17 @@ func startDaemon() (*Daemon, error) {
 	return d, nil
 }
 
+func haveMacAdmin() bool {
+	c, err := capability.NewPid(0)
+	if err != nil {
+		return false
+	}
+	if c.Get(capability.EFFECTIVE, capability.CAP_MAC_ADMIN) {
+		return true
+	}
+	return false
+}
+
 func (d *Daemon) Init() error {
 	/* Setup logging */
 	if shared.Log == nil {
@@ -508,6 +608,36 @@ func (d *Daemon) Init() error {
 			log.Ctx{"path": shared.VarPath("")})
 	}
 
+	/* Detect user namespaces */
+	runningInUserns = shared.RunningInUserNS()
+
+	/* Detect apparmor support */
+	if aaEnabled && os.Getenv("LXD_SECURITY_APPARMOR") == "false" {
+		aaEnabled = false
+		shared.Log.Warn("Per-container AppArmor profiles have been manually disabled")
+	}
+
+	if aaEnabled && !shared.IsDir("/sys/kernel/security/apparmor") {
+		aaEnabled = false
+		shared.Log.Warn("Per-container AppArmor profiles disabled because of lack of kernel support")
+	}
+
+	if aaEnabled && !haveMacAdmin() {
+		shared.Log.Warn("Per-container AppArmor profiles are disabled because mac_admin capability is missing.")
+		aaEnabled = false
+	}
+
+	_, err := exec.LookPath("apparmor_parser")
+	if aaEnabled && err != nil {
+		aaEnabled = false
+		shared.Log.Warn("Per-container AppArmor profiles disabled because 'apparmor_parser' couldn't be found")
+	}
+
+	if aaEnabled && runningInUserns {
+		aaEnabled = false
+		shared.Log.Warn("Per-container AppArmor profiles disabled because LXD is running inside a user namespace")
+	}
+
 	/* Get the list of supported architectures */
 	var architectures = []int{}
 
@@ -567,7 +697,7 @@ func (d *Daemon) Init() error {
 	d.IdmapSet, err = shared.DefaultIdmapSet()
 	if err != nil {
 		shared.Log.Warn("Error reading idmap", log.Ctx{"err": err.Error()})
-		shared.Log.Warn("Operations requiring idmap will not be available")
+		shared.Log.Warn("Only privileged containers will be able to run")
 	} else {
 		shared.Log.Info("Default uid/gid map:")
 		for _, lxcmap := range d.IdmapSet.ToLxcString() {
@@ -576,11 +706,7 @@ func (d *Daemon) Init() error {
 	}
 
 	/* Initialize the database */
-	if !d.IsMock {
-		err = initializeDbObject(d, shared.VarPath("lxd.db"))
-	} else {
-		err = initializeDbObject(d, ":memory:")
-	}
+	err = initializeDbObject(d, shared.VarPath("lxd.db"))
 	if err != nil {
 		return err
 	}
@@ -632,8 +758,10 @@ func (d *Daemon) Init() error {
 		}
 
 		/* Restart containers */
-		containersRestart(d)
-		containersWatch(d)
+		go func() {
+			containersWatch(d)
+			containersRestart(d)
+		}()
 
 		/* Setup the TLS authentication */
 		certf, keyf, err := readMyCert()
@@ -746,10 +874,10 @@ func (d *Daemon) Init() error {
 
 		tcpl, err := tls.Listen("tcp", listenAddr, tlsConfig)
 		if err != nil {
-			return fmt.Errorf("cannot listen on https socket: %v", err)
+			shared.Log.Error("cannot listen on https socket, skipping...", log.Ctx{"err": err})
+		} else {
+			sockets = append(sockets, Socket{Socket: tcpl, CloseOnExit: true})
 		}
-
-		sockets = append(sockets, Socket{Socket: tcpl, CloseOnExit: true})
 	}
 
 	if !d.IsMock {
@@ -865,6 +993,8 @@ func (d *Daemon) ConfigKeyIsValid(key string) bool {
 		return true
 	case "storage.lvm_thinpool_name":
 		return true
+	case "storage.zfs_pool_name":
+		return true
 	case "images.remote_cache_expiry":
 		return true
 	}
