@@ -219,8 +219,15 @@ func dbProfileConfigAdd(tx *sql.Tx, id int64, config map[string]string) error {
 	defer stmt.Close()
 
 	for k, v := range config {
+		if k == "raw.lxc" {
+			err := validateRawLxc(config["raw.lxc"])
+			if err != nil {
+				return err
+			}
+		}
+
 		if !ValidContainerConfigKey(k) {
-			return fmt.Errorf("Bad key: %s\n", k)
+			return fmt.Errorf("Bad key: %s", k)
 		}
 		_, err = stmt.Exec(id, k, v)
 		if err != nil {
