@@ -11,15 +11,15 @@ import (
 const DB_FIXTURES string = `
     INSERT INTO containers (name, architecture, type) VALUES ('thename', 1, 1);
     INSERT INTO profiles (name) VALUES ('theprofile');
-    INSERT INTO containers_profiles (container_id, profile_id) VALUES (1, 2);
+    INSERT INTO containers_profiles (container_id, profile_id) VALUES (1, 3);
     INSERT INTO containers_config (container_id, key, value) VALUES (1, 'thekey', 'thevalue');
     INSERT INTO containers_devices (container_id, name, type) VALUES (1, 'somename', 1);
     INSERT INTO containers_devices_config (key, value, container_device_id) VALUES ('configkey', 'configvalue', 1);
     INSERT INTO images (fingerprint, filename, size, architecture, creation_date, expiry_date, upload_date) VALUES ('fingerprint', 'filename', 1024, 0,  1431547174,  1431547175,  1431547176);
     INSERT INTO images_aliases (name, image_id, description) VALUES ('somealias', 1, 'some description');
     INSERT INTO images_properties (image_id, type, key, value) VALUES (1, 0, 'thekey', 'some value');
-    INSERT INTO profiles_config (profile_id, key, value) VALUES (2, 'thekey', 'thevalue');
-    INSERT INTO profiles_devices (profile_id, name, type) VALUES (2, 'devicename', 1);
+    INSERT INTO profiles_config (profile_id, key, value) VALUES (3, 'thekey', 'thevalue');
+    INSERT INTO profiles_devices (profile_id, name, type) VALUES (3, 'devicename', 1);
     INSERT INTO profiles_devices_config (profile_device_id, key, value) VALUES (2, 'devicekey', 'devicevalue');
     `
 
@@ -31,7 +31,7 @@ func createTestDb(t *testing.T) (db *sql.DB) {
 	}
 
 	var err error
-	d := &Daemon{}
+	d := &Daemon{IsMock: true}
 	err = initializeDbObject(d, ":memory:")
 	db = d.db
 
@@ -117,7 +117,7 @@ func Test_deleting_a_profile_cascades_on_related_tables(t *testing.T) {
 	}
 
 	// Make sure there are 0 container_profiles entries left.
-	statements = `SELECT count(*) FROM containers_profiles;`
+	statements = `SELECT count(*) FROM containers_profiles WHERE profile_id = 3;`
 	err = db.QueryRow(statements).Scan(&count)
 
 	if count != 0 {
@@ -125,7 +125,7 @@ func Test_deleting_a_profile_cascades_on_related_tables(t *testing.T) {
 	}
 
 	// Make sure there are 0 profiles_devices entries left.
-	statements = `SELECT count(*) FROM profiles_devices WHERE profile_id != 1;`
+	statements = `SELECT count(*) FROM profiles_devices WHERE profile_id == 3;`
 	err = db.QueryRow(statements).Scan(&count)
 
 	if count != 0 {
@@ -133,7 +133,7 @@ func Test_deleting_a_profile_cascades_on_related_tables(t *testing.T) {
 	}
 
 	// Make sure there are 0 profiles_config entries left.
-	statements = `SELECT count(*) FROM profiles_config;`
+	statements = `SELECT count(*) FROM profiles_config WHERE profile_id == 3;`
 	err = db.QueryRow(statements).Scan(&count)
 
 	if count != 0 {
@@ -141,7 +141,7 @@ func Test_deleting_a_profile_cascades_on_related_tables(t *testing.T) {
 	}
 
 	// Make sure there are 0 profiles_devices_config entries left.
-	statements = `SELECT count(*) FROM profiles_devices_config WHERE profile_device_id != 1;`
+	statements = `SELECT count(*) FROM profiles_devices_config WHERE profile_device_id == 3;`
 	err = db.QueryRow(statements).Scan(&count)
 
 	if count != 0 {
@@ -189,7 +189,7 @@ func Test_initializing_db_is_indempotent(t *testing.T) {
 	var err error
 
 	// This calls "createDb" once already.
-	d := &Daemon{}
+	d := &Daemon{IsMock: true}
 	err = initializeDbObject(d, ":memory:")
 	db = d.db
 
@@ -198,7 +198,7 @@ func Test_initializing_db_is_indempotent(t *testing.T) {
 	// Let's call it a second time.
 	err = createDb(db)
 	if err != nil {
-		t.Error("The database schema is not indempotent.")
+		t.Errorf("The database schema is not indempotent, err='%s'", err)
 	}
 }
 
@@ -221,22 +221,14 @@ func Test_running_dbUpdateFromV6_adds_on_delete_cascade(t *testing.T) {
 	// Upgrading the database schema with updateFromV6 adds ON DELETE CASCADE
 	// to sqlite tables that require it, and conserve the data.
 
-	var db *sql.DB
 	var err error
 	var count int
-	var statements string
 
-	d := &Daemon{}
+	d := &Daemon{IsMock: true}
 	err = initializeDbObject(d, ":memory:")
-	db = d.db
+	defer d.db.Close()
 
-	defer db.Close()
-
-	if err != nil {
-		t.Error(err)
-	}
-
-	statements = `
+	statements := `
 CREATE TABLE IF NOT EXISTS containers (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
     name VARCHAR(255) NOT NULL,
@@ -254,20 +246,21 @@ CREATE TABLE IF NOT EXISTS containers_config (
     FOREIGN KEY (container_id) REFERENCES containers (id),
     UNIQUE (container_id, key)
 );
+
 INSERT INTO containers (name, architecture, type) VALUES ('thename', 1, 1);
 INSERT INTO containers_config (container_id, key, value) VALUES (1, 'thekey', 'thevalue');`
 
-	_, err = db.Exec(statements)
+	_, err = d.db.Exec(statements)
 	if err != nil {
 		t.Error(err)
 	}
 
 	// Run the upgrade from V6 code
-	err = dbUpdateFromV6(db)
+	err = dbUpdateFromV6(d.db)
 
 	// Make sure the inserted data is still there.
 	statements = `SELECT count(*) FROM containers_config;`
-	err = db.QueryRow(statements).Scan(&count)
+	err = d.db.QueryRow(statements).Scan(&count)
 
 	if count != 1 {
 		t.Fatalf("There should be exactly one entry in containers_config! There are %d.", count)
@@ -276,14 +269,14 @@ INSERT INTO containers_config (container_id, key, value) VALUES (1, 'thekey', 't
 	// Drop the container.
 	statements = `DELETE FROM containers WHERE name = 'thename';`
 
-	_, err = db.Exec(statements)
+	_, err = d.db.Exec(statements)
 	if err != nil {
 		t.Errorf("Error deleting container! %s", err)
 	}
 
 	// Make sure there are 0 container_profiles entries left.
 	statements = `SELECT count(*) FROM containers_profiles;`
-	err = db.QueryRow(statements).Scan(&count)
+	err = d.db.QueryRow(statements).Scan(&count)
 
 	if count != 0 {
 		t.Errorf("Deleting a container didn't delete the profile association! There are %d left", count)
@@ -373,7 +366,7 @@ INSERT INTO containers_config (container_id, key, value) VALUES (1, 'thekey', 't
 
 	// The "foreign key" on containers_config now points to nothing.
 	// Let's run the schema upgrades.
-	d := &Daemon{}
+	d := &Daemon{IsMock: true}
 	d.db = db
 	err = dbUpdate(d, 1)
 
@@ -405,7 +398,7 @@ func Test_dbImageGet_finds_image_for_fingerprint(t *testing.T) {
 	db = createTestDb(t)
 	defer db.Close()
 
-	result, err = dbImageGet(db, "fingerprint", false)
+	result, err = dbImageGet(db, "fingerprint", false, false)
 
 	if err != nil {
 		t.Fatal(err)
@@ -439,7 +432,7 @@ func Test_dbImageGet_for_missing_fingerprint(t *testing.T) {
 	db = createTestDb(t)
 	defer db.Close()
 
-	_, err = dbImageGet(db, "unknown", false)
+	_, err = dbImageGet(db, "unknown", false, false)
 
 	if err != sql.ErrNoRows {
 		t.Fatal("Wrong err type returned")
@@ -538,7 +531,7 @@ func Test_dbProfileConfigGet(t *testing.T) {
 	db = createTestDb(t)
 	defer db.Close()
 
-	_, err = db.Exec("INSERT INTO profiles_config (profile_id, key, value) VALUES (2, 'something', 'something else');")
+	_, err = db.Exec("INSERT INTO profiles_config (profile_id, key, value) VALUES (3, 'something', 'something else');")
 
 	result, err = dbProfileConfigGet(db, "theprofile")
 	if err != nil {
@@ -565,7 +558,6 @@ func Test_dbContainerProfilesGet(t *testing.T) {
 
 	expected = []string{"theprofile"}
 	result, err = dbContainerProfilesGet(db, 1)
-
 	if err != nil {
 		t.Fatal(err)
 	}
