@@ -7,6 +7,8 @@ import (
 	"fmt"
 	"os"
 	"os/exec"
+	"path/filepath"
+	"strconv"
 	"strings"
 	"syscall"
 
@@ -14,29 +16,158 @@ import (
 
 	"github.com/lxc/lxd/shared"
 	"gopkg.in/lxc/go-lxc.v2"
+
+	log "gopkg.in/inconshreveable/log15.v2"
 )
 
-func addBlockDev(dev string) ([]string, error) {
-	stat := syscall.Stat_t{}
-	err := syscall.Stat(dev, &stat)
-	if err != nil {
-		return []string{}, err
+func devGetOptions(d shared.Device) (string, error) {
+	opts := []string{"bind", "create=file"}
+	if d["uid"] != "" {
+		u, err := strconv.Atoi(d["uid"])
+		if err != nil {
+			return "", err
+		}
+		opts = append(opts, fmt.Sprintf("uid=%d", u))
 	}
-	k := "lxc.cgroup.devices.allow"
-	v := fmt.Sprintf("b %d:%d rwm", uint(stat.Rdev/256), uint(stat.Rdev%256))
-	line := []string{k, v}
-	return line, err
+	if d["gid"] != "" {
+		g, err := strconv.Atoi(d["gid"])
+		if err != nil {
+			return "", err
+		}
+		opts = append(opts, fmt.Sprintf("gid=%d", g))
+	}
+	if d["mode"] != "" {
+		m, err := devModeOct(d["mode"])
+		if err != nil {
+			return "", err
+		}
+		opts = append(opts, fmt.Sprintf("mode=%0d", m))
+	} else {
+		opts = append(opts, "mode=0660")
+	}
+
+	return strings.Join(opts, ","), nil
 }
 
-func deviceToLxc(d shared.Device) ([][]string, error) {
+func modeHasRead(mode int) bool {
+	if mode&0444 != 0 {
+		return true
+	}
+	return false
+}
+
+func modeHasWrite(mode int) bool {
+	if mode&0222 != 0 {
+		return true
+	}
+	return false
+}
+
+func devModeOct(strmode string) (int, error) {
+	if strmode == "" {
+		return 0660, nil
+	}
+	i, err := strconv.ParseInt(strmode, 8, 32)
+	if err != nil {
+		return 0, fmt.Errorf("Bad device mode: %s", strmode)
+	}
+	return int(i), nil
+}
+
+func devModeString(strmode string) (string, error) {
+	i, err := devModeOct(strmode)
+	if err != nil {
+		return "", err
+	}
+	mode := "m"
+	if modeHasRead(i) {
+		mode = mode + "r"
+	}
+	if modeHasWrite(i) {
+		mode = mode + "w"
+	}
+	return mode, nil
+}
+
+func getDev(path string) (int, int, error) {
+	stat := syscall.Stat_t{}
+	err := syscall.Stat(path, &stat)
+	if err != nil {
+		return 0, 0, err
+	}
+	major := int(stat.Rdev / 256)
+	minor := int(stat.Rdev % 256)
+	return major, minor, nil
+}
+
+func deviceCgroupInfo(dev shared.Device) (string, error) {
+	var err error
+
+	t := dev["type"]
+	switch t {
+	case "unix-char":
+		t = "c"
+	case "unix-block":
+		t = "b"
+	default: // internal error - look at how we were called
+		return "", fmt.Errorf("BUG: bad device type %s", dev["type"])
+	}
+
+	var major, minor int
+	if dev["major"] == "" && dev["minor"] == "" {
+		devname := dev["path"]
+		if !filepath.IsAbs(devname) {
+			devname = filepath.Join("/", devname)
+		}
+		major, minor, err = getDev(devname)
+		if err != nil {
+			return "", err
+		}
+	} else if dev["major"] != "" && dev["minor"] != "" {
+		major, err = strconv.Atoi(dev["major"])
+		if err != nil {
+			return "", err
+		}
+		minor, err = strconv.Atoi(dev["minor"])
+		if err != nil {
+			return "", err
+		}
+	} else {
+		return "", fmt.Errorf("Both major and minor must be supplied for devices")
+	}
+
+	mode, err := devModeString(dev["mode"])
+	if err != nil {
+		return "", err
+	}
+	devcg := fmt.Sprintf("%s %d:%d %s", t, major, minor, mode)
+	return devcg, nil
+}
+
+/*
+ * unixDevCgroup only grabs the cgroup devices.allow statement
+ * we need.  We'll add a mount.entry to bind mount the actual
+ * device later.
+ */
+func unixDevCgroup(dev shared.Device) ([][]string, error) {
+	devcg, err := deviceCgroupInfo(dev)
+	if err != nil {
+		return [][]string{}, err
+	}
+	entry := []string{"lxc.cgroup.devices.allow", devcg}
+	return [][]string{entry}, nil
+}
+
+func deviceToLxc(cntPath string, d shared.Device) ([][]string, error) {
 	switch d["type"] {
 	case "unix-char":
-		return nil, fmt.Errorf("Not implemented")
+		return unixDevCgroup(d)
 	case "unix-block":
-		return nil, fmt.Errorf("Not implemented")
+		return unixDevCgroup(d)
+
 	case "nic":
 		if d["nictype"] != "bridged" && d["nictype"] != "" {
-			return nil, fmt.Errorf("Bad nic type: %s\n", d["nictype"])
+			return nil, fmt.Errorf("Bad nic type: %s", d["nictype"])
 		}
 		var l1 = []string{"lxc.network.type", "veth"}
 		var lines = [][]string{l1}
@@ -68,22 +199,15 @@ func deviceToLxc(d shared.Device) ([][]string, error) {
 		} else {
 			p = d["path"]
 		}
-		/* TODO - check whether source is a disk, loopback, btrfs subvol, etc */
-		/* for now we only handle directory bind mounts */
 		source := d["source"]
-		fstype := "none"
 		options := []string{}
-		var err error
 		if shared.IsBlockdevPath(d["source"]) {
-			fstype, err = shared.BlockFsDetect(d["source"])
+			l, err := mountTmpBlockdev(cntPath, d)
 			if err != nil {
-				return nil, fmt.Errorf("Error setting up %s: %s\n", d["name"], err)
-			}
-			l, err := addBlockDev(d["source"])
-			if err != nil {
-				return nil, fmt.Errorf("Error adding blockdev: %s\n", err)
+				return nil, fmt.Errorf("Error adding blockdev: %s", err)
 			}
 			configLines = append(configLines, l)
+			return configLines, nil
 		} else if shared.IsDir(source) {
 			options = append(options, "bind")
 			options = append(options, "create=dir")
@@ -103,7 +227,7 @@ func deviceToLxc(d shared.Device) ([][]string, error) {
 		if opts == "" {
 			opts = "defaults"
 		}
-		l := []string{"lxc.mount.entry", fmt.Sprintf("%s %s %s %s 0 0", source, p, fstype, opts)}
+		l := []string{"lxc.mount.entry", fmt.Sprintf("%s %s %s %s 0 0", source, p, "none", opts)}
 		configLines = append(configLines, l)
 		return configLines, nil
 	case "none":
@@ -126,7 +250,7 @@ func dbDeviceTypeToString(t int) (string, error) {
 	case 4:
 		return "unix-block", nil
 	default:
-		return "", fmt.Errorf("Invalid device type %d\n", t)
+		return "", fmt.Errorf("Invalid device type %d", t)
 	}
 }
 
@@ -143,7 +267,7 @@ func deviceTypeToDbType(t string) (int, error) {
 	case "unix-block":
 		return 4, nil
 	default:
-		return -1, fmt.Errorf("Invalid device type %s\n", t)
+		return -1, fmt.Errorf("Invalid device type %s", t)
 	}
 }
 
@@ -225,7 +349,7 @@ func validDeviceConfig(t, k, v string) bool {
 func tempNic() string {
 	randBytes := make([]byte, 4)
 	rand.Read(randBytes)
-	return "lxd" + hex.EncodeToString(randBytes)
+	return "veth" + hex.EncodeToString(randBytes)
 }
 
 func inList(l []string, s string) bool {
@@ -255,12 +379,12 @@ func nextUnusedNic(c container) string {
 	}
 }
 
-func setupNic(c container, d map[string]string) (string, error) {
+func setupNic(tx *sql.Tx, c container, name string, d map[string]string) (string, error) {
 	if d["nictype"] != "bridged" {
-		return "", fmt.Errorf("Unsupported nic type: %s\n", d["nictype"])
+		return "", fmt.Errorf("Unsupported nic type: %s", d["nictype"])
 	}
 	if d["parent"] == "" {
-		return "", fmt.Errorf("No bridge given\n")
+		return "", fmt.Errorf("No bridge given")
 	}
 	if d["name"] == "" {
 		d["name"] = nextUnusedNic(c)
@@ -273,12 +397,47 @@ func setupNic(c container, d map[string]string) (string, error) {
 	if err != nil {
 		return "", err
 	}
+
 	err = exec.Command("brctl", "addif", d["parent"], n1).Run()
 	if err != nil {
 		removeInterface(n2)
 		return "", err
 	}
 
+	key := fmt.Sprintf("volatile.%s.hwaddr", name)
+	config := c.ConfigGet()
+	hwaddr := config[key]
+
+	if hwaddr == "" {
+		if d["hwaddr"] != "" {
+			hwaddr, err = generateMacAddr(d["hwaddr"])
+			if err != nil {
+				return "", err
+			}
+		} else {
+			hwaddr, err = generateMacAddr("00:16:3e:xx:xx:xx")
+			if err != nil {
+				return "", err
+			}
+		}
+
+		if hwaddr != d["hwaddr"] {
+			stmt := `INSERT OR REPLACE into containers_config (container_id, key, value) VALUES (?, ?, ?)`
+			_, err = tx.Exec(stmt, c.IDGet(), key, hwaddr)
+
+			if err != nil {
+				removeInterface(n2)
+				return "", err
+			}
+		}
+	}
+
+	err = exec.Command("ip", "link", "set", "dev", n2, "address", hwaddr).Run()
+	if err != nil {
+		removeInterface(n2)
+		return "", err
+	}
+
 	return n2, nil
 }
 
@@ -335,6 +494,33 @@ func txUpdateNic(tx *sql.Tx, cId int, devname string, nicname string) error {
 	return err
 }
 
+func (c *containerLXD) DetachUnixDev(dev shared.Device) error {
+	cginfo, err := deviceCgroupInfo(dev)
+	if err != nil {
+		return err
+	}
+	c.c.SetCgroupItem("devices.remove", cginfo)
+	pid := c.c.InitPid()
+	if pid == -1 { // container not running
+		return nil
+	}
+	pidstr := fmt.Sprintf("%d", pid)
+	if err := exec.Command(os.Args[0], "forkumount", pidstr, dev["path"]).Run(); err != nil {
+		shared.Log.Warn("Error unmounting device", log.Ctx{"Error": err})
+		return err
+	}
+	if err := os.Remove(fmt.Sprintf("/proc/%d/root/%s", pid, dev["path"])); err != nil {
+		shared.Log.Warn("Error removing device", log.Ctx{"Error": err})
+		return err
+	}
+
+	return nil
+}
+
+func (c *containerLXD) AttachUnixDev(dev shared.Device) error {
+	return c.setupUnixDev(dev)
+}
+
 /*
  * Given a running container and a list of devices before and after a
  * config change, update the devices in the container.
@@ -350,13 +536,17 @@ func devicesApplyDeltaLive(tx *sql.Tx, c container, preDevList shared.Devices, p
 		switch dev["type"] {
 		case "nic":
 			if dev["name"] == "" {
-				return fmt.Errorf("Do not know a name for the nic for device %s\n", key)
+				return fmt.Errorf("Do not know a name for the nic for device %s", key)
 			}
 			if err := detachInterface(c, dev["name"]); err != nil {
 				return fmt.Errorf("Error removing device %s (nic %s) from container %s: %s", key, dev["name"], c.NameGet(), err)
 			}
 		case "disk":
 			return c.DetachMount(dev)
+		case "unix-block":
+			return c.DetachUnixDev(dev)
+		case "unix-char":
+			return c.DetachUnixDev(dev)
 		}
 	}
 
@@ -366,7 +556,7 @@ func devicesApplyDeltaLive(tx *sql.Tx, c container, preDevList shared.Devices, p
 		switch dev["type"] {
 		case "nic":
 			var tmpName string
-			if tmpName, err = setupNic(c, dev); err != nil {
+			if tmpName, err = setupNic(tx, c, key, dev); err != nil {
 				return fmt.Errorf("Unable to create nic %s for container %s: %s", dev["name"], c.NameGet(), err)
 			}
 			if err := lxContainer.AttachInterface(tmpName, dev["name"]); err != nil {
@@ -383,19 +573,12 @@ func devicesApplyDeltaLive(tx *sql.Tx, c container, preDevList shared.Devices, p
 				return fmt.Errorf("no source or destination given")
 			}
 			return c.AttachMount(dev)
+		case "unix-block":
+			return c.AttachUnixDev(dev)
+		case "unix-char":
+			return c.AttachUnixDev(dev)
 		}
 	}
 
 	return nil
 }
-
-func validateConfig(c container, devs shared.Devices) error {
-	for _, dev := range devs {
-		if dev["type"] == "disk" && shared.IsBlockdevPath(dev["source"]) {
-			if !c.IsPrivileged() {
-				return fmt.Errorf("Only privileged containers may mount block devices")
-			}
-		}
-	}
-	return nil
-}
