@@ -3,13 +3,34 @@ package main
 import (
 	"io/ioutil"
 	"os"
-	"path"
+	"sync"
 	"testing"
 
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
 )
 
+func mockStartDaemon() (*Daemon, error) {
+	d := &Daemon{
+		IsMock:                true,
+		imagesDownloading:     map[string]chan bool{},
+		imagesDownloadingLock: sync.RWMutex{},
+	}
+
+	if err := d.Init(); err != nil {
+		return nil, err
+	}
+
+	// Call this after Init so we have a log object.
+	storageConfig := make(map[string]interface{})
+	d.Storage = &storageLogWrapper{w: &storageMock{d: d}}
+	if _, err := d.Storage.Init(storageConfig); err != nil {
+		return nil, err
+	}
+
+	return d, nil
+}
+
 type lxdTestSuite struct {
 	suite.Suite
 	d      *Daemon
@@ -18,16 +39,11 @@ type lxdTestSuite struct {
 }
 
 func (suite *lxdTestSuite) SetupSuite() {
-	cwd, err := os.Getwd()
-	if err != nil {
-		os.Exit(1)
-	}
-
-	suite.tmpdir, err = ioutil.TempDir(
-		path.Join(path.Dir(cwd), "test"), "lxd_testrun_")
+	tmpdir, err := ioutil.TempDir("", "lxd_testrun_")
 	if err != nil {
 		os.Exit(1)
 	}
+	suite.tmpdir = tmpdir
 
 	if err := os.Setenv("LXD_DIR", suite.tmpdir); err != nil {
 		os.Exit(1)
