@@ -18,6 +18,7 @@ const (
 	filesystemSuperMagicExt4  = 0xEF53
 	filesystemSuperMagicXfs   = 0x58465342
 	filesystemSuperMagicNfs   = 0x6969
+	filesystemSuperMagicZfs   = 0x2fc12fc1
 )
 
 /*
@@ -35,6 +36,8 @@ func filesystemDetect(path string) (string, error) {
 	switch fs.Type {
 	case filesystemSuperMagicBtrfs:
 		return "btrfs", nil
+	case filesystemSuperMagicZfs:
+		return "zfs", nil
 	case filesystemSuperMagicTmpfs:
 		return "tmpfs", nil
 	case filesystemSuperMagicExt4:
@@ -44,6 +47,7 @@ func filesystemDetect(path string) (string, error) {
 	case filesystemSuperMagicNfs:
 		return "nfs", nil
 	default:
+		shared.Debugf("Unknown backing filesystem type: 0x%x", fs.Type)
 		return string(fs.Type), nil
 	}
 }
@@ -73,33 +77,12 @@ func storageRsyncCopy(source string, dest string) (string, error) {
 	return string(output), err
 }
 
-func storageUnprivUserAclSet(c container, dpath string) error {
-	idmapset := c.IdmapSetGet()
-
-	if idmapset == nil {
-		return nil
-	}
-	uid, _ := idmapset.ShiftIntoNs(0, 0)
-	switch uid {
-	case -1:
-		shared.Debugf("No root id mapping")
-		return nil
-	case 0:
-		return nil
-	}
-	acl := fmt.Sprintf("%d:rx", uid)
-	output, err := exec.Command("setfacl", "-m", acl, dpath).CombinedOutput()
-	if err != nil {
-		shared.Debugf("Setfacl failed:\n%s", string(output))
-	}
-	return err
-}
-
 // storageType defines the type of a storage
 type storageType int
 
 const (
 	storageTypeBtrfs storageType = iota
+	storageTypeZfs
 	storageTypeLvm
 	storageTypeDir
 	storageTypeMock
@@ -109,6 +92,8 @@ func storageTypeToString(sType storageType) string {
 	switch sType {
 	case storageTypeBtrfs:
 		return "btrfs"
+	case storageTypeZfs:
+		return "zfs"
 	case storageTypeLvm:
 		return "lvm"
 	case storageTypeMock:
@@ -166,6 +151,12 @@ func newStorageWithConfig(d *Daemon, sType storageType, config map[string]interf
 		}
 
 		s = &storageLogWrapper{w: &storageBtrfs{d: d}}
+	case storageTypeZfs:
+		if d.Storage != nil && d.Storage.GetStorageType() == storageTypeZfs {
+			return d.Storage, nil
+		}
+
+		s = &storageLogWrapper{w: &storageZfs{d: d}}
 	case storageTypeLvm:
 		if d.Storage != nil && d.Storage.GetStorageType() == storageTypeLvm {
 			return d.Storage, nil
@@ -192,19 +183,20 @@ func storageForFilename(d *Daemon, filename string) (storage, error) {
 		return nil, fmt.Errorf("couldn't detect filesystem for '%s': %v", filename, err)
 	}
 
-	lvLinkPath := filename + ".lv"
-	if shared.PathExists(lvLinkPath) {
+	if shared.PathExists(filename + ".lv") {
 		storageType = storageTypeLvm
-		lvPath, err := os.Readlink(lvLinkPath)
+		lvPath, err := os.Readlink(filename + ".lv")
 		if err != nil {
-			return nil, fmt.Errorf("couldn't read link dest for '%s': %v", lvLinkPath, err)
+			return nil, fmt.Errorf("couldn't read link dest for '%s': %v", filename+".lv", err)
 		}
 		vgname := filepath.Base(filepath.Dir(lvPath))
 		config["vgName"] = vgname
-
-	} else if filesystem == "btrfs" {
+	} else if shared.PathExists(filename + ".zfs") {
+		storageType = storageTypeZfs
+	} else if shared.PathExists(filename+".btrfs") || filesystem == "btrfs" {
 		storageType = storageTypeBtrfs
 	}
+
 	return newStorageWithConfig(d, storageType, config)
 }
 
@@ -266,26 +258,29 @@ func (ss *storageShared) shiftRootfs(c container) error {
 }
 
 func (ss *storageShared) setUnprivUserAcl(c container, destPath string) error {
+	idmapset := c.IdmapSetGet()
 
-	if !c.IsPrivileged() {
-		err := storageUnprivUserAclSet(c, destPath)
+	// Skip for privileged containers
+	if idmapset == nil {
+		return nil
+	}
+
+	// Make sure the map is valid. Skip if container uid 0 == host uid 0
+	uid, _ := idmapset.ShiftIntoNs(0, 0)
+	switch uid {
+	case -1:
+		return fmt.Errorf("Container doesn't have a uid 0 in its map")
+	case 0:
+		return nil
+	}
+
+	// Attempt to set a POSIX ACL first. Fallback to chmod if the fs doesn't support it.
+	acl := fmt.Sprintf("%d:rx", uid)
+	_, err := exec.Command("setfacl", "-m", acl, destPath).CombinedOutput()
+	if err != nil {
+		_, err := exec.Command("chmod", "+x", destPath).CombinedOutput()
 		if err != nil {
-			ss.log.Error(
-				"Adding acl for container root: falling back to chmod",
-				log.Ctx{"destPath": destPath})
-
-			output, err := exec.Command(
-				"chmod", "+x", destPath).CombinedOutput()
-
-			if err != nil {
-				ss.log.Error(
-					"chmoding the container root",
-					log.Ctx{
-						"destPath": destPath,
-						"output":   string(output)})
-
-				return err
-			}
+			return fmt.Errorf("Failed to chmod the container path.")
 		}
 	}
 
