@@ -4,6 +4,8 @@ import (
 	"bytes"
 	"encoding/json"
 	"fmt"
+
+	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 type Jmap map[string]interface{}
@@ -51,9 +53,11 @@ func (m Jmap) GetBool(key string) (bool, error) {
 func DebugJson(r *bytes.Buffer) {
 	pretty := &bytes.Buffer{}
 	if err := json.Indent(pretty, r.Bytes(), "\t", "\t"); err != nil {
-		Debugf("error indenting json: ", err)
+		Log.Error("error indenting json", log.Ctx{"err": err})
 		return
 	}
 
-	Debugf("\n\t%s", pretty.String())
+	// Print the JSON without the last "\n"
+	str := pretty.String()
+	Log.Debug(fmt.Sprintf("\n\t%s", str[0:len(str)-1]))
 }
