@@ -36,7 +36,7 @@ func SetLogger(syslog string, logfile string, verbose bool, debug bool) error {
 	// FileHandler
 	if logfile != "" {
 		if !PathExists(filepath.Dir(logfile)) {
-			return fmt.Errorf("Log file path doesn't exist: %s\n", filepath.Dir(logfile))
+			return fmt.Errorf("Log file path doesn't exist: %s", filepath.Dir(logfile))
 		}
 
 		if !debug {
@@ -65,6 +65,14 @@ func SetLogger(syslog string, logfile string, verbose bool, debug bool) error {
 		} else {
 			handlers = append(handlers, log.StderrHandler)
 		}
+	} else {
+		handlers = append(
+			handlers,
+			log.LvlFilterHandler(
+				log.LvlWarn,
+				log.StderrHandler,
+			),
+		)
 	}
 
 	Log.SetHandler(log.MultiHandler(handlers...))
